"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const _ = require("lodash");
const cliHighlight = require("cli-highlight");
const express = require("express");
const fs = require("fs-extra");
const os = require("os");
const path = require("path");
const sanitizeFilename = require('sanitize-filename');
const contracts_1 = require("../../contracts");
const http_1 = require("../../http");
const util_1 = require("../../util");
const API_MODULE_PROPERTY = Symbol('API_MODULE_PROPERTY');
const BOOTSTRAP_FILENAME = '_bootstrap.js';
const SHUTDOWN_FILENAME = '_shutdown.js';
/**
 * Api command.
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = "Runs an Express.js based REST API from current directory.";
    }
    /** @inheritdoc */
    async execute(ctx) {
        const { app, https, server } = http_1.createHttpServer({
            forceHttp: ctx.args['h'] || ctx.args['http'],
        });
        app.use(async (req, res, next) => {
            res.header('X-Powered-By', 'e.GO CLI');
            res.header('X-Tm-Mk', '1979-09-05 23:09');
            res.header('Last-Modified', (new Date()).toUTCString());
            return next();
        });
        const API_ROUTE = express.Router();
        let baseUrl;
        {
            if (_.isNil(ctx.args['bu']) && _.isNil(ctx.args['base-url'])) {
                baseUrl = '/api';
            }
            else {
                baseUrl = util_1.toStringSafe(ctx.args['bu']);
                if ('' === baseUrl.trim()) {
                    baseUrl = util_1.toStringSafe(ctx.args['base-url']);
                }
            }
            baseUrl = util_1.toStringSafe(baseUrl)
                .split(path.sep).join('/')
                .trim();
            if (!baseUrl.startsWith('/')) {
                baseUrl = '/' + baseUrl;
            }
            while (baseUrl.endsWith('/')) {
                baseUrl = baseUrl.substr(0, baseUrl.length - 1)
                    .trim();
            }
            baseUrl = '/' + baseUrl.split('/')
                .map(x => x.trim())
                .filter(x => '' !== x)
                .join('/');
        }
        const ON_SHUTDOWN = await this.setupAPIRoute(ctx, API_ROUTE, app);
        app.use(baseUrl, API_ROUTE);
        // TCP port
        let port = parseInt(util_1.toStringSafe(ctx.args['p'])
            .trim());
        if (isNaN(port)) {
            port = parseInt(util_1.toStringSafe(ctx.args['port'])
                .trim());
        }
        if (isNaN(port)) {
            port = 8080; // default
        }
        // try start host ...
        await util_1.withSpinnerAsync(`Starting API host on port ${port} ...`, (spinner) => {
            return new Promise((resolve, reject) => {
                try {
                    server.once('error', (err) => {
                        reject(err);
                    });
                    server.listen(port, () => {
                        spinner.text = `API host now running on port ${port}`;
                        resolve();
                    });
                }
                catch (e) {
                    reject(e);
                }
            });
        });
        // output all base URLs
        util_1.writeLine();
        util_1.writeLine(`You can now access the API from the following base URLs:`);
        util_1.writeLine();
        {
            const SCHEME = https ?
                'https' : 'http';
            const NETWORK_INTERFACES = os.networkInterfaces();
            const LIST_OF_IFNAMES = Object.keys(NETWORK_INTERFACES).sort((x, y) => {
                return util_1.compareValuesBy(x, y, n => {
                    return util_1.toStringSafe(n)
                        .toLowerCase()
                        .trim();
                });
            });
            for (const IFNAME of LIST_OF_IFNAMES) {
                const IFACES = NETWORK_INTERFACES[IFNAME].filter(x => {
                    return !x.internal;
                }).filter(x => {
                    const ADDR = util_1.toStringSafe(x.address)
                        .toLowerCase()
                        .trim();
                    if ('IPv4' === x.family) {
                        return !/^(127\.[\d.]+|[0:]+1|localhost)$/.test(ADDR);
                    }
                    if ('IPv6' === x.family) {
                        return '::1' !== ADDR;
                    }
                    return true;
                }).sort((x, y) => {
                    return util_1.compareValuesBy(x, y, (i) => {
                        return 'IPv4' === i.family ? 0 : 1;
                    });
                });
                IFACES.forEach(x => {
                    util_1.writeLine(`    ${SCHEME}://${'IPv6' === x.family ? '[' : ''}${x.address}${'IPv6' === x.family ? ']' : ''}:${port}${baseUrl}${'/' !== baseUrl ? '/' : ''}`);
                });
            }
            util_1.writeLine();
            util_1.writeLine(`    ${SCHEME}://localhost:${port}${baseUrl}${'/' !== baseUrl ? '/' : ''}`);
            util_1.writeLine(`    ${SCHEME}://127.0.0.1:${port}${baseUrl}${'/' !== baseUrl ? '/' : ''}`);
        }
        // wait for ENTER
        util_1.writeLine();
        util_1.writeLine();
        await util_1.waitForEnter('Press <ENTER> to stop ...');
        await ON_SHUTDOWN();
        server.close((err) => {
            if (err) {
                util_1.writeErrLine(err);
                ctx.exit(1);
            }
            else {
                ctx.exit();
            }
        });
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Options:`);
        util_1.writeLine(` --b, --bearer      # Defines an optional 'Bearer' token for 'Authorization' request header.`);
        util_1.writeLine(` --bu, --base-url   # Defines the base URL for the API. Default: '/api'`);
        util_1.writeLine(` -h, --http         # Force in-secure HTTP or not.`);
        util_1.writeLine(` --p, --port        # The custom TCP port. Default: 8080`);
        util_1.writeLine(` --pwd, --password  # The password for basic authorization.`);
        util_1.writeLine(` --r, --root        # Custom root directory. Default: current working directory`);
        util_1.writeLine(` --u, --user        # The user name for basic authorization.`);
        util_1.writeLine(` -v, --verbose      # Verbose output.`);
        util_1.writeLine();
        util_1.writeLine(`Examples:  ego api`);
        util_1.writeLine(`           ego api --port=23979`);
        util_1.writeLine(`           ego api --bearer=footoken`);
        util_1.writeLine(`           ego api --user=tanja --password=19790905`);
        util_1.writeLine();
        util_1.writeLine();
        util_1.writeLine(`Example endpoint:`);
        util_1.writeLine();
        util_1.writeLine(`Create an 'index.js' file and use the following skeleton:`);
        util_1.writeLine(cliHighlight.highlight(`
    // for other methods like 'post' or 'delete'
    // simply implement them by their names in upper case
    // characters ('POST' and/or 'DELETE', e.g.)
    exports.GET = async function(req, res) {
        // for 'req' and 'res'  =>  https://expressjs.com/

        // s. https://egodigital.github.io/ego-cli/interfaces/_contracts_.commandexecutecontext.html
        const CONTEXT = this;

        return res.status(200)
            .send('Hello, e.GO!');
    };

    // handle any HTTP method
    // and/or use as fallback
    exports.request = async function(req, res) {
        return res.status(501)
            .send();
    };
`, {
            language: 'javascript',
        }));
        util_1.writeLine(`To create another endpoint, like '/foo/bar', use one of the following file paths:`);
        util_1.writeLine(`  * /foo/bar.js`);
        util_1.writeLine(`  * /foo/bar/index.js`);
        util_1.writeLine();
        util_1.writeLine(`Files with leading _ will be ignored, if you want to use them for endpoints.`);
        util_1.writeLine();
        util_1.writeLine(`To define startup logic, create a '_bootstrap.js' at the root of your API directory:`);
        util_1.writeLine(cliHighlight.highlight(`
    exports.execute = async (context, apiRouter, app) => {
        // context    =>  https://egodigital.github.io/ego-cli/interfaces/_contracts_.commandexecutecontext.html
        // apiRouter  =>  https://expressjs.com/en/guide/routing.html
        // app        =>  https://expressjs.com/en/starter/hello-world.html
    };
`, {
            language: 'javascript',
        }));
        util_1.writeLine(`You can do the same thing with a '_shutdown.js' file.`);
    }
    async setupAPIRoute(ctx, route, app) {
        const VERBOSE = ctx.args['v'] || ctx.args['verbose'];
        // -r or --root
        let rootDir = util_1.toStringSafe(ctx.args['r']);
        if ('' === rootDir.trim()) {
            rootDir = util_1.toStringSafe(ctx.args['root']);
        }
        if ('' === rootDir.trim()) {
            rootDir = ctx.cwd;
        }
        if (!path.isAbsolute(rootDir)) {
            rootDir = path.join(ctx.cwd, rootDir);
        }
        rootDir = path.resolve(rootDir);
        const FIND_SCRIPT_FILE = async (req) => {
            let normalizedPath = req.path
                .split(path.sep).join('/')
                .split('/')
                .map(x => x.trim())
                .filter(x => '' !== x)
                .map(x => sanitizeFilename(x))
                .join('/');
            if (!normalizedPath.startsWith('/')) {
                normalizedPath = '/' + normalizedPath;
            }
            const POSSIBLE_FILES = [
                path.join(normalizedPath, 'index.js')
            ];
            if ('' !== path.basename(normalizedPath).trim()) {
                POSSIBLE_FILES.unshift(path.join(path.dirname(normalizedPath), path.basename(normalizedPath) + '.js'));
            }
            for (const PF of POSSIBLE_FILES) {
                const FULL_PATH = path.resolve(path.join(rootDir, PF));
                if ((rootDir !== FULL_PATH) &&
                    (!FULL_PATH.startsWith(rootDir + path.sep))) {
                    // invalid path
                    continue;
                }
                if (!(await util_1.exists(FULL_PATH))) {
                    continue;
                }
                const STATS = await fs.stat(FULL_PATH);
                if (STATS.isFile()) {
                    return require.resolve(FULL_PATH);
                }
            }
            return false;
        };
        const LOAD_SCRIPT_MODULE = async (scriptFile) => {
            scriptFile = require.resolve(scriptFile);
            delete require.cache[scriptFile];
            return require(scriptFile);
        };
        const PRINT_REQUEST_ERROR = (err, req) => {
            if (VERBOSE) {
                util_1.writeErrLine();
                util_1.writeErrLine(`API REQUEST ERROR [${req.method} :: ${req.path}] => ${util_1.toStringSafe(err)}`);
            }
        };
        if (VERBOSE) {
            route.use(async (req, res, next) => {
                util_1.writeLine();
                util_1.writeLine(`API REQUEST [${req.method} :: ${req.path}] => ` + JSON.stringify({
                    headers: req.headers,
                    query: req.query,
                }, null, 2));
                return next();
            });
        }
        // bearer token
        let bearerToken = util_1.toStringSafe(ctx.args['b'])
            .trim();
        if ('' === bearerToken) {
            bearerToken = util_1.toStringSafe(ctx.args['bearer'])
                .trim();
        }
        if ('' !== bearerToken) {
            // check for bearer token
            route.use(async (req, res, next) => {
                const AUTHORIZATION = util_1.toStringSafe(req.headers['authorization'])
                    .trim();
                if (AUTHORIZATION.toLowerCase().startsWith('bearer')) {
                    const TOKEN = AUTHORIZATION.substr(6)
                        .trim();
                    if (TOKEN === bearerToken) {
                        return next(); // matches
                    }
                }
                return res.status(401)
                    .send();
            });
        }
        // username and password for basic auth
        let username = util_1.toStringSafe(ctx.args['u']);
        if ('' === username.trim()) {
            username = util_1.toStringSafe(ctx.args['user']);
        }
        let password = util_1.toStringSafe(ctx.args['pwd']);
        if ('' === password.trim()) {
            password = util_1.toStringSafe(ctx.args['password']);
        }
        username = util_1.toStringSafe(username)
            .toLowerCase()
            .trim();
        password = util_1.toStringSafe(password);
        if ('' !== username || '' !== password.trim()) {
            // check for user and password
            route.use(async (req, res, next) => {
                try {
                    const AUTHORIZATION = util_1.toStringSafe(req.headers['authorization'])
                        .trim();
                    if (AUTHORIZATION.toLowerCase().startsWith('basic')) {
                        const BASE64_USERNAME_AND_PASSWORD = AUTHORIZATION.substr(5)
                            .trim();
                        if ('' !== BASE64_USERNAME_AND_PASSWORD) {
                            const USERNAME_AND_PASSWORD = Buffer.from(BASE64_USERNAME_AND_PASSWORD, 'base64').toString('utf8');
                            let u;
                            let p;
                            const SEP = USERNAME_AND_PASSWORD.indexOf(':');
                            if (SEP > -1) {
                                u = USERNAME_AND_PASSWORD.substr(0, SEP);
                                p = USERNAME_AND_PASSWORD.substr(SEP + 1);
                            }
                            else {
                                u = USERNAME_AND_PASSWORD;
                            }
                            u = util_1.toStringSafe(u)
                                .toLowerCase()
                                .trim();
                            p = util_1.toStringSafe(p);
                            if (u === username && p === password) {
                                return next(); // user & password match
                            }
                        }
                    }
                }
                catch (e) {
                    PRINT_REQUEST_ERROR(e, req);
                }
                return res.status(401)
                    .header('WWW-Authenticate', 'Basic realm="API by e.GO CLI"')
                    .send();
            });
        }
        // check if script (module) exists
        route.use(async (req, res, next) => {
            const SCRIPT_FILE = await FIND_SCRIPT_FILE(req);
            if (false === SCRIPT_FILE) {
                return res.status(404)
                    .send();
            }
            req[API_MODULE_PROPERTY] = await LOAD_SCRIPT_MODULE(SCRIPT_FILE);
            return next();
        });
        const BOOTSTRAP_FILE = path.join(rootDir, BOOTSTRAP_FILENAME);
        if (await util_1.exists(BOOTSTRAP_FILE)) {
            const BOOTSTRAP_MODULE = await LOAD_SCRIPT_MODULE(BOOTSTRAP_FILE);
            if (BOOTSTRAP_MODULE.execute) {
                // run bootstrap file
                await Promise.resolve(BOOTSTRAP_MODULE.execute(ctx, route, app));
            }
        }
        // handle request
        route.all('/*', async function (req, res) {
            try {
                const API_MODULE = req[API_MODULE_PROPERTY];
                const POSSIBLE_METHODS = [
                    util_1.toStringSafe(req.method)
                        .toUpperCase()
                        .trim(),
                    'request',
                ];
                let endpoint = false;
                for (const PM of POSSIBLE_METHODS) {
                    const VALUE = API_MODULE[PM];
                    if ('function' === typeof VALUE) {
                        endpoint = VALUE;
                        break;
                    }
                }
                if (false === endpoint) {
                    // not implemented
                    return res.status(405)
                        .send();
                }
                return await Promise.resolve(endpoint.apply(ctx, arguments));
            }
            catch (e) {
                PRINT_REQUEST_ERROR(e, req);
                return res.status(500)
                    .header('Content-type', 'text/plain; charset=utf-8')
                    .send(Buffer.from(util_1.toStringSafe(e), 'utf8'));
            }
        });
        return async () => {
            const SHUTDOWN_FILE = path.join(rootDir, SHUTDOWN_FILENAME);
            if (await util_1.exists(SHUTDOWN_FILE)) {
                const SHUTDOWN_MODULE = await LOAD_SCRIPT_MODULE(SHUTDOWN_FILE);
                if (SHUTDOWN_MODULE.execute) {
                    // run shutdown file
                    await Promise.resolve(SHUTDOWN_MODULE.execute(ctx, route, app));
                }
            }
        };
    }
}
exports.EgoCommand = EgoCommand;
//# sourceMappingURL=index.js.map