"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const _ = require("lodash");
const fs = require("fs-extra");
const path = require("path");
const contracts_1 = require("../../contracts");
const util_1 = require("../../util");
/**
 * Build command.
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = "Builds the current project.";
    }
    /** @inheritdoc */
    async execute(ctx) {
        const PACKAGE_JSON_FILE = path.resolve(path.join(ctx.cwd, 'package.json'));
        if (!(await util_1.exists(PACKAGE_JSON_FILE))) {
            console.warn(`'package.json' file not found!`);
            ctx.exit(1);
        }
        const PACKAGE_JSON = JSON.parse(await fs.readFile(PACKAGE_JSON_FILE, 'utf8'));
        if (_.isNil(PACKAGE_JSON.scripts)) {
            console.warn(`No scripts defined in 'package.json'!`);
            ctx.exit(2);
        }
        if (_.isNil(PACKAGE_JSON.scripts.build)) {
            console.warn(`No 'build' script defined in 'package.json'!`);
            ctx.exit(3);
        }
        const USE_YARN = ctx.args['y'] ||
            ctx.args['yarn'] ||
            ctx.get('yarn');
        if (USE_YARN) {
            await util_1.withSpinnerAsync(`Executing 'yarn run build' ...`, async (spinner) => {
                await util_1.spawnAsync('yarn', ['run', 'build'], {
                    cwd: ctx.cwd,
                    stdio: util_1.getSTDIO(ctx),
                });
                spinner.text = `'yarn run build' executed`;
            });
        }
        else {
            await util_1.withSpinnerAsync(`Executing 'npm run build' ...`, async (spinner) => {
                await util_1.spawnAsync('npm', ['run', 'build'], {
                    cwd: ctx.cwd,
                    stdio: util_1.getSTDIO(ctx),
                });
                spinner.text = `'npm run build' executed`;
            });
        }
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Options:`);
        util_1.writeLine(` -v, --verbose  # Verbose output.`);
        util_1.writeLine(` -y, --yarn     # Use yarn instead.`);
        util_1.writeLine();
        util_1.writeLine(`Config:`);
        util_1.writeLine(` yarn   # Use yarn instead.`);
        util_1.writeLine();
        util_1.writeLine(`Examples:  ego build`);
        util_1.writeLine(`           ego build -y`);
    }
}
exports.EgoCommand = EgoCommand;
//# sourceMappingURL=index.js.map