"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const lib = require("https");
const contracts_1 = require("../../contracts");
const util_1 = require("../../util");
/**
 * Chuck command
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = 'Print a random Chuck Norris joke. Jep it is true! Just try it 😎';
        /** @inheritdoc */
        this.syntax = '[options]'; // Syntax, that is shown in help screen
        this.getJoke = function (url) {
            // return new pending promise
            return new Promise((resolve, reject) => {
                const request = lib.get(url, (response) => {
                    // handle http errors
                    if (response.statusCode < 200 || response.statusCode > 299) {
                        reject(new Error('Failed to load page, status code: ' + response.statusCode));
                    }
                    // temporary data holder
                    const body = [];
                    // on every content chunk, push it to the data array
                    response.on('data', (chunk) => body.push(chunk));
                    // we are done, resolve promise with those joined chunks
                    response.on('end', () => resolve(body.join('')));
                });
                // handle connection errors of the request
                request.on('error', (err) => reject(err));
            });
        };
    }
    /** @inheritdoc */
    async execute(context) {
        let url = "https://api.icndb.com/jokes/";
        if (context.args['i'] || context.args['id']) {
            const id = context.args.i ? context.args.i : context.args.id;
            if (!isNaN(parseFloat(id)) && isFinite(id)) {
                url += "" + id;
            }
            else {
                util_1.writeErrLine("DUDE! You have to input a number like in this example: ego chuck -i 104");
                context.exit(1);
            }
        }
        else {
            url += "random";
        }
        if (context.args['n'] || context.args['name']) {
            const name = (context.args.n ? context.args.n : context.args.name) + "";
            const split = name.split('-');
            if (split.length === 2) {
                url += '?firstName=' + encodeURIComponent(split[0]) + "&lastName=" + encodeURIComponent(split[1]);
                url += "&escape=javascript";
            }
            else if (split.length === 1 && split[0].length > 1) {
                url += '?firstName=' + encodeURIComponent(split[0]) + "&lastName= ";
                url += "&escape=javascript";
            }
            else {
                util_1.writeErrLine("DUDE! We need a first AND a last name! Divided by \"-\" i.e. ego chuck -n Ole-Mustermann");
                context.exit(2);
            }
        }
        else {
            url += "?escape=javascript";
        }
        if (!context.args['e'] && !context.args['explicit']) {
            url += "&exclude=[explicit]";
        }
        await util_1.withSpinnerAsync(`Loading joke ...`, async (spinner) => {
            try {
                let response = JSON.parse(await this.getJoke(url));
                if (response && response.type && response.type === "success") {
                    spinner.text = `👊 [${response.value.id}] ${response.value.joke} 👊`;
                }
                else {
                    this.handleError(spinner, "Invalid Response");
                }
            }
            catch (error) {
                this.handleError(spinner, error);
            }
        });
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Options:`);
        util_1.writeLine(` -e, --explicit  # Include explicit jokes.`);
        util_1.writeLine(` --i, --id       # Print a specific joke.`);
        util_1.writeLine(` --n, --name     # You need a little confidence boost?`);
        util_1.writeLine(`                 # No problem, just write down your name.`);
        util_1.writeLine(`                 # If you provide a first and a last name please divide them by "-".`);
        util_1.writeLine();
        util_1.writeLine(`Examples:  ego chuck`);
        util_1.writeLine(`           ego chuck --name Ole-Mustermann`);
        util_1.writeLine(`           ego chuck --n Ole`);
        util_1.writeLine(`           ego chuck --i 15`);
        util_1.writeLine(`           ego chuck --id 15 --n Ole-Mustermann`);
    }
    handleError(spinner, error) {
        spinner.fail(`OH BOY, something really bad happend, even Chuck Norris can't do anything about it ... Maybe this can help him: '${util_1.toStringSafe(error)}'`);
    }
}
exports.EgoCommand = EgoCommand;
//# sourceMappingURL=index.js.map