"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const _ = require("lodash");
const cliProgress = require("cli-progress");
const csvParser = require("csv-parser");
const fs = require("fs-extra");
const path = require("path");
const progressStream = require("progress-stream");
const contracts_1 = require("../../contracts");
const util_1 = require("../../util");
/**
 * Csv-Split command.
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = "Splits one or more (huge) CSV file(s) into separates parts.";
        /** @inheritdoc */
        this.syntax = '[FILES+] [OPTIONS*]';
    }
    /** @inheritdoc */
    async execute(ctx) {
        const FILES = util_1.asArray(ctx.args['_']).map(a => util_1.toStringSafe(a))
            .filter(a => '' !== a.trim());
        if (!FILES.length) {
            console.warn('Please define a least one CSV file, you would like to handle!');
            ctx.exit(1);
        }
        for (const F of FILES) {
            const BAR = new cliProgress.Bar({
                barsize: 20,
                format: trimFilenameIfNeeded(F) + '    [{bar}] {percentage}%  |  ETA: {eta}s',
                barIncompleteChar: ' ',
            });
            try {
                BAR.start(100, 0);
                await this.splitCSVFile(ctx, F, (progress) => {
                    BAR.update(progress.percentage);
                });
            }
            finally {
                BAR.stop();
            }
        }
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Options:`);
        util_1.writeLine(` --e, --escape      # The escape string to use. Default: quote string`);
        util_1.writeLine(` --enc, --encoding  # The encoding to use. Default: utf8`);
        util_1.writeLine(` --l, --lines       # Maximum number of lines per file. Default: 10000`);
        util_1.writeLine(` -lf, --line-feed   # Use LF instead of CRLF for new lines. Default: (false)`);
        util_1.writeLine(` --o, --out         # The output directory. Default: current working directory`);
        util_1.writeLine(` --q, --quote       # Specifies a single-character string to denote a quoted string. Default: ,`);
        util_1.writeLine(` --s, --separator   # The column separator to use. Default: ,`);
        util_1.writeLine();
        util_1.writeLine(`Relative paths will be mapped to the current working directory.`);
        util_1.writeLine();
        util_1.writeLine(`Examples:  ego csv-split my-file.csv`);
        util_1.writeLine(`           ego csv-split my-file-1.csv my-file-2.csv --lines=5979`);
    }
    async splitCSVFile(ctx, file, onProgess) {
        if (!path.isAbsolute(file)) {
            file = path.join(ctx.cwd, file);
        }
        file = path.resolve(file);
        const STATS = await fs.stat(file);
        // -o / --out
        let outDir = util_1.toStringSafe(ctx.args['o']);
        if ('' === outDir.trim()) {
            outDir = util_1.toStringSafe(ctx.args['out']);
        }
        if ('' === outDir.trim()) {
            outDir = ctx.cwd;
        }
        if (!path.isAbsolute(outDir)) {
            outDir = path.join(ctx.cwd, outDir);
        }
        outDir = path.resolve(outDir);
        // -l / --lines
        let maxLinesPerFile = parseInt(util_1.toStringSafe(ctx.args['l'])
            .trim());
        if (isNaN(maxLinesPerFile)) {
            maxLinesPerFile = parseInt(util_1.toStringSafe(ctx.args['lines'])
                .trim());
        }
        if (isNaN(maxLinesPerFile)) {
            maxLinesPerFile = 10000;
        }
        // -s / --separator
        let separator = util_1.toStringSafe(ctx.args['s']);
        if ('' === separator.trim()) {
            separator = util_1.toStringSafe(ctx.args['separator']);
        }
        if ('' === separator.trim()) {
            separator = ','; // default
        }
        // -q / --quote
        let quote;
        if (_.isNil(ctx.args['q']) && _.isNil(ctx.args['quote'])) {
            quote = '"';
        }
        else {
            if (!_.isNil(ctx.args['q'])) {
                quote = util_1.toStringSafe(ctx.args['q']);
            }
            else if (!_.isNil(ctx.args['quote'])) {
                quote = util_1.toStringSafe(ctx.args['quote']);
            }
        }
        // -e / --escape
        let escape;
        if (_.isNil(ctx.args['e']) && _.isNil(ctx.args['escape'])) {
            escape = quote;
        }
        else {
            if (!_.isNil(ctx.args['e'])) {
                escape = util_1.toStringSafe(ctx.args['e']);
            }
            else if (!_.isNil(ctx.args['escape'])) {
                escape = util_1.toStringSafe(ctx.args['escape']);
            }
        }
        // -enc / --encoding
        let encoding = util_1.toStringSafe(ctx.args['enc'])
            .toLowerCase()
            .trim();
        if ('' === encoding) {
            encoding = util_1.toStringSafe(ctx.args['encoding'])
                .toLowerCase()
                .trim();
        }
        if ('' === encoding) {
            encoding = 'utf8'; // default
        }
        const NEW_LINE = ctx.args['lf'] || ctx.args['line-feed']
            ? "\n" : "\r\n";
        let chunks = -1;
        let currentFileName;
        let lines = -1;
        await (() => {
            return new Promise(async (resolve, reject) => {
                try {
                    fs.createReadStream(file)
                        .pipe(progressStream({
                        length: STATS.size,
                    }))
                        .on('progress', onProgess)
                        .pipe(csvParser({
                        escape: escape,
                        newline: NEW_LINE,
                        quote: quote,
                        separator: separator,
                    }))
                        .once('error', (err) => {
                        reject(err);
                    })
                        .once('end', () => {
                        resolve(undefined);
                    })
                        .on('data', (data) => {
                        ++lines;
                        if (0 === lines % maxLinesPerFile) { // new file?
                            ++chunks;
                            const EXT = path.extname(file);
                            const BASE_FILENAME = path.basename(file, EXT);
                            // find unique name
                            let fileName = BASE_FILENAME + '-' + chunks;
                            let index = -1;
                            do {
                                const FULL_PATH = path.join(outDir, fileName + EXT);
                                if (!fs.existsSync(FULL_PATH)) {
                                    currentFileName = FULL_PATH;
                                    break;
                                }
                                // try next ...
                                ++index;
                                fileName = BASE_FILENAME + '-' + chunks + '-' + index;
                            } while (true);
                            // header
                            fs.appendFileSync(currentFileName, Object.keys(data)
                                .join(separator), encoding);
                        }
                        // data row
                        fs.appendFileSync(currentFileName, Object.values(data)
                            .join(separator), encoding);
                    });
                }
                catch (e) {
                    reject(e);
                }
            });
        })();
    }
}
exports.EgoCommand = EgoCommand;
function trimFilenameIfNeeded(fileName, maxSize = 20) {
    const EXT = path.extname(fileName);
    let baseName = path.basename(fileName, EXT)
        .trim();
    if (baseName.length > maxSize) {
        baseName = baseName.substr(0, maxSize) + '\u2026';
    }
    return baseName + EXT;
}
//# sourceMappingURL=index.js.map