"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const cliHighlight = require("cli-highlight");
const got_1 = require("got");
const contracts_1 = require("../../contracts");
const util_1 = require("../../util");
/**
 * Devops-Items-Update command.
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = "Updates one or more Azure DevOps work items.";
        /** @inheritdoc */
        this.syntax = '[WORK_ITEM_ID]+ [options]';
    }
    /** @inheritdoc */
    async execute(ctx) {
        // DevOps user
        const USER = util_1.toStringSafe(ctx.get('devops_user'))
            .toLowerCase()
            .trim();
        if ('' === USER) {
            console.warn(`Please setup ${util_1.colorize('devops_user')} config value, by executing ${util_1.colorize('ego set devops_user "tanja.m"')}`);
            ctx.exit(1);
        }
        // Personal Access Token
        const PAT = util_1.toStringSafe(ctx.get('devops_pat'))
            .trim();
        if ('' === PAT) {
            console.warn(`Please setup ${util_1.colorize('devops_pat')} config value, by executing ${util_1.colorize('ego set devops_pat "xyz"')}`);
            ctx.exit(2);
        }
        // organization
        const ORG = util_1.toStringSafe(ctx.get('devops_org'))
            .trim();
        if ('' === ORG) {
            console.warn(`Please setup ${util_1.colorize('devops_org')} config value, by executing ${util_1.colorize('ego set devops_org "microsoft"')}`);
            ctx.exit(3);
        }
        // new state
        let state = util_1.toStringSafe(ctx.args['s']).trim();
        if ('' === state) {
            state = util_1.toStringSafe(ctx.args['state']).trim();
        }
        // new user, the item should be assigned to
        let assignedTo = util_1.toStringSafe(ctx.args['a']).trim();
        if ('' === assignedTo) {
            assignedTo = util_1.toStringSafe(ctx.args['assign-to']).trim();
        }
        // work item IDs
        const ID_LIST = util_1.asArray(ctx.args['_'])
            .map(x => parseInt(util_1.toStringSafe(x).trim()))
            .filter(x => !isNaN(x) && x);
        if (!ID_LIST.length) {
            console.warn(`Please define at least one ID of a work item!`);
            ctx.exit(4);
        }
        for (const ID of ID_LIST) {
            await util_1.withSpinnerAsync(`Updating work item ${ID} ...`, async (spinner) => {
                try {
                    const RESPONSE = await got_1.default.get(`https://dev.azure.com/${encodeURIComponent(ORG)}/_apis/wit/workitems?ids=${encodeURIComponent(ID.toString())}&api-version=5.1`, {
                        headers: {
                            'Authorization': `Basic ${Buffer.from(`${USER}:${PAT}`, 'utf8').toString('base64')}`,
                        },
                    });
                    if (200 === RESPONSE.statusCode) {
                        const WORK_ITEMS = JSON.parse(RESPONSE.body);
                        if (WORK_ITEMS.count) {
                            for (const V of WORK_ITEMS.value) {
                                const OPERATIONS = [];
                                if ('' !== state) {
                                    OPERATIONS.push({
                                        "op": "replace",
                                        "path": "/fields/System.State",
                                        "value": state,
                                    });
                                }
                                if ('' !== assignedTo) {
                                    OPERATIONS.push({
                                        "op": "replace",
                                        "path": "/fields/System.AssignedTo",
                                        "value": assignedTo,
                                    });
                                }
                                if (OPERATIONS.length) {
                                    const UPDATE_RESPONSE = await got_1.default.patch(`https://dev.azure.com/${encodeURIComponent(ORG)}/_apis/wit/workitems/${encodeURIComponent(V.id.toString())}?api-version=5.1`, {
                                        headers: {
                                            'Authorization': `Basic ${Buffer.from(`${USER}:${PAT}`, 'utf8').toString('base64')}`,
                                            'Content-type': 'application/json-patch+json',
                                        },
                                        body: JSON.stringify(OPERATIONS),
                                        throwHttpErrors: false,
                                    });
                                    if (200 === UPDATE_RESPONSE.statusCode) {
                                        spinner.text = `Work item ${ID} updated`;
                                    }
                                    else {
                                        // failed
                                        spinner.fail(`Unexpected response for work item ${ID}: [${UPDATE_RESPONSE.statusCode}] '${UPDATE_RESPONSE.statusMessage}'

${cliHighlight.highlight(JSON.stringify(JSON.parse(UPDATE_RESPONSE.body), null, 2), {
                                            "language": "javascript"
                                        })}`);
                                    }
                                }
                                else {
                                    spinner.text = `Work item ${ID} updated`;
                                }
                            }
                        }
                        else {
                            spinner.warn(`Empty result for work item ${ID}!`);
                        }
                    }
                    else {
                        spinner.fail(`Unexpected response for work item ${ID}: [${RESPONSE.statusCode}] '${RESPONSE.statusMessage}'`);
                    }
                }
                catch (e) {
                    spinner.fail(`Could not update work item ${ID}: '${util_1.toStringSafe(e)}'`);
                }
            });
        }
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Options:`);
        util_1.writeLine(` --a, --assign-to  # Sets the new user, the item should be assigned to.`);
        util_1.writeLine(` --s, --state      # The name of the new state, like 'Active'.`);
        util_1.writeLine();
        util_1.writeLine(`Config:`);
        util_1.writeLine(` devops_org   # The name of the organization,`);
        util_1.writeLine(` devops_pat   # The Personal Access Token for accessing the API.`);
        util_1.writeLine(` devops_user  # The Azure DevOps user name.`);
        util_1.writeLine();
        util_1.writeLine(`Examples:  ego devops-items-update 23979 --state=Active`);
        util_1.writeLine(`           ego devops-items-update 5979 --assigned-to="Marcel Kloubert" --state=Active`);
    }
}
exports.EgoCommand = EgoCommand;
//# sourceMappingURL=index.js.map