"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const _ = require("lodash");
const cliHighlight = require("cli-highlight");
const fs = require("fs-extra");
const contracts_1 = require("../../contracts");
const storage_1 = require("../../storage");
const util_1 = require("../../util");
/**
 * Get command.
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = "Lists one or more config value(s).";
        /** @inheritdoc */
        this.syntax = '[REGEX_FILTER*] [options]';
    }
    /** @inheritdoc */
    async execute(ctx) {
        await ctx.queue.add(async () => {
            const GLOBAL = ctx.args['g'] || ctx.args['global'];
            const LOCAL = ctx.args['l'] || ctx.args['local'];
            const TABLE = ctx.args['t'] || ctx.args['table'];
            const STORAGES = [];
            const ADD_GLOBAL_STORAGE = () => {
                STORAGES.push(storage_1.getStorage());
            };
            const ADD_LOCAL_STORAGE = () => {
                const LOCAL_STORAGE_FILE_PATH = ctx.getFullPath(contracts_1.EGO_FOLDER + '/' + contracts_1.STORAGE_FILE);
                if (fs.existsSync(LOCAL_STORAGE_FILE_PATH)) {
                    const STAT = fs.statSync(LOCAL_STORAGE_FILE_PATH);
                    if (STAT.isFile()) {
                        STORAGES.push(JSON.parse(fs.readFileSync(LOCAL_STORAGE_FILE_PATH, 'utf8')));
                    }
                }
            };
            if (!GLOBAL && !LOCAL) {
                ADD_GLOBAL_STORAGE();
                ADD_LOCAL_STORAGE();
            }
            else {
                if (GLOBAL) {
                    ADD_GLOBAL_STORAGE();
                }
                if (LOCAL) {
                    ADD_LOCAL_STORAGE();
                }
            }
            let keyFilter;
            const FILTERS = util_1.asArray(ctx.args['_'])
                .map(x => util_1.toStringSafe(x))
                .filter(x => '' !== x.trim());
            if (FILTERS.length) {
                keyFilter = (key) => {
                    for (const F of FILTERS) {
                        const REGEX = new RegExp(F, 'i');
                        if (REGEX.test(key)) {
                            return true;
                        }
                    }
                    return false;
                };
            }
            else {
                keyFilter = () => true;
            }
            let keysFound = false;
            let filteredStorage = {};
            for (const S of STORAGES) {
                if (!_.isObjectLike(S)) {
                    continue;
                }
                for (const KEY in S) {
                    if (keyFilter(KEY)) {
                        filteredStorage[storage_1.normalizeStorageKey(KEY)] = S[KEY];
                        keysFound = true;
                    }
                }
            }
            filteredStorage = util_1.sortObjectByKeys(filteredStorage);
            if (TABLE) {
                for (const KEY in filteredStorage) {
                    util_1.writeLine(`${KEY}\t${JSON.stringify(filteredStorage[KEY])}`);
                }
            }
            else {
                if (keysFound) {
                    util_1.writeLine(cliHighlight.highlight(JSON.stringify(filteredStorage, null, 2), {
                        'language': 'json',
                    }));
                }
            }
        });
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Options:`);
        util_1.writeLine(` -g, --global  # List only global values, if possible.`);
        util_1.writeLine(` -l, --local   # List only local values, if possible.`);
        util_1.writeLine(` -t, --table   # Outputs value(s) in a simple list.`);
        util_1.writeLine();
        util_1.writeLine(`Examples:  ego get`);
        util_1.writeLine(`           ego get email`);
        util_1.writeLine(`           ego get username email -tl`);
    }
}
exports.EgoCommand = EgoCommand;
//# sourceMappingURL=index.js.map