"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const contracts_1 = require("../../contracts");
const git_1 = require("../../git");
const util_1 = require("../../util");
/**
 * Git-Checkout command.
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = "Checks out (to) a branch.";
        /** @inheritdoc */
        this.syntax = 'BRANCH [options]';
    }
    /** @inheritdoc */
    async execute(ctx) {
        const TARGET_BRANCH = util_1.toStringSafe(ctx.args['_'][0]).trim();
        if ('' === TARGET_BRANCH) {
            console.warn('Please define the branch, which you would like to checkout!');
            ctx.exit(1);
        }
        const MERGE_WITH_CURRENT = ctx.args['m'] || ctx.args['merge'];
        let currentBranch;
        if (MERGE_WITH_CURRENT) {
            await util_1.withSpinnerAsync('Detecting current branch ...', async (spinner) => {
                currentBranch = git_1.getCurrentGitBranch({
                    cwd: ctx.cwd,
                    verbose: ctx.verbose,
                });
                spinner.text = `Branch: '${currentBranch}'`;
            });
        }
        await util_1.withSpinnerAsync(`Checking out branch '${TARGET_BRANCH}' ...`, async (spinner) => {
            await util_1.spawnAsync('git', ['checkout', TARGET_BRANCH], {
                cwd: ctx.cwd,
                stdio: util_1.getSTDIO(ctx),
            });
            spinner.text = `Branch '${TARGET_BRANCH}' checked out`;
        });
        if (MERGE_WITH_CURRENT) {
            await util_1.withSpinnerAsync(`Merging with branch '${currentBranch}' ...`, async (spinner) => {
                await util_1.spawnAsync('git', ['merge', currentBranch], {
                    cwd: ctx.cwd,
                    stdio: util_1.getSTDIO(ctx),
                });
                spinner.text = `Merged with branch '${TARGET_BRANCH}'`;
            });
        }
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Options:`);
        util_1.writeLine(` -m, --merge    # After checkout, merge the current branch with the new one.`);
        util_1.writeLine(` -v, --verbose  # Verbose output.`);
        util_1.writeLine();
        util_1.writeLine(`Examples:  ego git-checkout dev`);
        util_1.writeLine(`           ego git-checkout dev -m`);
    }
}
exports.EgoCommand = EgoCommand;
//# sourceMappingURL=index.js.map