"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const Enumerable = require("node-enumerable");
const inquirer = require("inquirer");
const contracts_1 = require("../../contracts");
const git_1 = require("../../git");
const util_1 = require("../../util");
/**
 * Git-delete command.
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = "Deletes local branches, except 'master' and the current one.";
        /** @inheritdoc */
        this.syntax = '[options]';
    }
    /** @inheritdoc */
    async execute(ctx) {
        const YES = ctx.args['y'] || ctx.args['yes'];
        let filterExpr = util_1.toStringSafe(ctx.args['f']);
        if ('' === filterExpr.trim()) {
            filterExpr = util_1.toStringSafe(ctx.args['filter']);
        }
        let filter = false;
        if ('' !== filterExpr.trim()) {
            filter = new RegExp(filterExpr, 'i');
        }
        let currentBranch;
        await util_1.withSpinnerAsync('Detecting current branch ...', async (spinner) => {
            currentBranch = git_1.getCurrentGitBranch({
                cwd: ctx.cwd,
                verbose: ctx.verbose,
            });
            spinner.text = `Branch: '${currentBranch}'`;
        });
        let allBranches;
        await util_1.withSpinnerAsync('Loading branches ...', async (spinner) => {
            allBranches = git_1.getGitBranches({
                cwd: ctx.cwd,
                verbose: ctx.verbose,
            });
            spinner.text = allBranches.length + ' branch(es) found';
        });
        // no 'master' and
        // not the current one
        const DELETABLE_BRANCHES = Enumerable.from(allBranches)
            .where(b => 'master' !== b)
            .where(b => currentBranch !== b)
            .where(b => {
            return filter ? filter.test(b) : true;
        })
            .distinct()
            .orderBy(b => util_1.toStringSafe(b).toLowerCase().trim())
            .toArray();
        if (!DELETABLE_BRANCHES.length) {
            return;
        }
        let selectedBranches;
        if (YES) {
            selectedBranches = DELETABLE_BRANCHES;
        }
        else {
            // let the user select the branch(es)
            const CHECKBOX_SELECTIONS = await inquirer.prompt({
                type: 'checkbox',
                name: 'branches_to_delete',
                message: 'Please select the branches, you would like to delete:',
                choices: DELETABLE_BRANCHES.map(db => {
                    return {
                        checked: true,
                        name: db,
                    };
                }),
            });
            if (CHECKBOX_SELECTIONS) {
                selectedBranches = CHECKBOX_SELECTIONS['branches_to_delete'];
            }
        }
        if (!selectedBranches) {
            return;
        }
        for (const B of selectedBranches) {
            await util_1.withSpinnerAsync(`Deleting branch '${B}' ...`, async (spinner) => {
                await util_1.spawnAsync('git', ['branch', '-D', B], {
                    cwd: ctx.cwd,
                    stdio: util_1.getSTDIO(ctx),
                });
                spinner.text = `Branch '${B}' deleted`;
            });
        }
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Options:`);
        util_1.writeLine(` -f, --filter  # Regular expression, which filters the branches by name.`);
        util_1.writeLine(`               # Filters are case-insensitive.`);
        util_1.writeLine(` -y, --yes     # Do not ask the user to confirm the operation.`);
        util_1.writeLine();
        util_1.writeLine(`Examples:  ego git-delete`);
        util_1.writeLine(`           ego git-delete -y`);
        util_1.writeLine(`           ego git-delete --format="^(feature\\/)"`);
    }
}
exports.EgoCommand = EgoCommand;
//# sourceMappingURL=index.js.map