"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const fs = require("fs-extra");
const path = require("path");
const contracts_1 = require("../../contracts");
const util_1 = require("../../util");
/**
 * Git-Export command.
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = "Clones a repository to the working directory and removes the '.git' subfolder.";
        /** @inheritdoc */
        this.syntax = 'URL [options]';
    }
    /** @inheritdoc */
    async execute(ctx) {
        const REPO_URL = util_1.toStringSafe(ctx.args['_'][0]).trim();
        if ('' === REPO_URL) {
            console.warn('Please define the URL of the repository, that should be exported!');
            ctx.exit(1);
        }
        await util_1.withSpinnerAsync(`Cloning repository '${REPO_URL}' ...`, async (spinner) => {
            await util_1.spawnAsync('git', ['clone', REPO_URL, '.'], {
                cwd: ctx.cwd,
                stdio: util_1.getSTDIO(ctx),
            });
            spinner.text = `Repository '${REPO_URL}' cloned`;
        });
        await util_1.withSpinnerAsync(`Removing '.git' folder ...`, async (spinner) => {
            const GIT_FOLDER = path.resolve(path.join(ctx.cwd, '.git'));
            if (await util_1.exists(GIT_FOLDER)) {
                const STAT = await fs.lstat(GIT_FOLDER);
                if (STAT.isDirectory()) {
                    await fs.remove(GIT_FOLDER);
                }
                else if (STAT.isSymbolicLink()) {
                    await fs.unlink(GIT_FOLDER);
                }
            }
            spinner.text = `'.git' folder removed`;
        });
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Options:`);
        util_1.writeLine(` -v, --verbose  # Verbose output.`);
        util_1.writeLine();
        util_1.writeLine(`Example:  ego git-export https://github.com/egodigital/generator-ego`);
    }
}
exports.EgoCommand = EgoCommand;
//# sourceMappingURL=index.js.map