"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const _ = require("lodash");
const path = require("path");
const contracts_1 = require("../../contracts");
const help_1 = require("../../help");
const scripts_1 = require("../../scripts");
const util_1 = require("../../util");
/**
 * Help command.
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = 'Shows the help screen for a command.';
        /** @inheritdoc */
        this.syntax = '[COMMAND]';
    }
    /** @inheritdoc */
    async execute(ctx) {
        let command;
        let commandName;
        if (ctx.args['_'].length < 1) {
            command = this;
            commandName = ctx.name;
        }
        else {
            const COMMAND_NAME = util_1.toStringSafe(ctx.args['_'][0])
                .toLowerCase()
                .trim();
            if ('' === COMMAND_NAME) {
                console.warn('No command defined!');
                ctx.exit(2);
            }
            if (!contracts_1.REGEX_COMMAND_NAME.test(COMMAND_NAME)) {
                console.warn(`Invalid command name ('${COMMAND_NAME}')!`);
                ctx.exit(3);
            }
            // bash script?
            if (scripts_1.executeShellScriptCommand(COMMAND_NAME, ['--?'])) {
                ctx.exit();
                return; // yes
            }
            const MODULE_FILE = path.resolve(path.join(__dirname, '../', COMMAND_NAME, 'index.js'));
            if (!(await util_1.exists(MODULE_FILE))) {
                await help_1.suggestCommand(COMMAND_NAME);
                ctx.exit(4);
            }
            const COMMAND_CLASS = require(MODULE_FILE).EgoCommand;
            if (_.isNil(COMMAND_CLASS)) {
                console.warn(`Command '${COMMAND_NAME}' not implemented!`);
                ctx.exit(5);
            }
            const COMMAND = new COMMAND_CLASS();
            if (_.isNil(COMMAND.showHelp)) {
                console.warn(`Command '${COMMAND_NAME}'.showHelp() not implemented!`);
                ctx.exit(6);
            }
            command = COMMAND;
            commandName = COMMAND_NAME;
        }
        util_1.writeLine(util_1.eGO(command.description));
        util_1.writeLine();
        util_1.writeLine(`Syntax:    ego ${commandName} ${command.syntax}`);
        util_1.writeLine();
        const CTX = {
            package: ctx.package,
        };
        const EXIT_CODE = parseInt(util_1.toStringSafe(await Promise.resolve(command.showHelp(CTX))).trim());
        if (isNaN(EXIT_CODE)) {
            ctx.exit();
        }
        else {
            ctx.exit(EXIT_CODE);
        }
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Examples:  ego help new`);
        util_1.writeLine(`           ego help`);
    }
}
exports.EgoCommand = EgoCommand;
//# sourceMappingURL=index.js.map