"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const fs = require("fs-extra");
const path = require("path");
const contracts_1 = require("../../contracts");
const util_1 = require("../../util");
/**
 * Init command.
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = "Initializes an '.ego' subfolder in the current directory for local based settings and data.";
    }
    /** @inheritdoc */
    async execute(ctx) {
        const FORCE = ctx.args['f'] || ctx.args['force'];
        const EGO_FOLDER_PATH = path.resolve(path.join(ctx.cwd, contracts_1.EGO_FOLDER));
        if (await util_1.exists(EGO_FOLDER_PATH)) {
            if (FORCE) {
                await util_1.withSpinnerAsync(`Removing existing '.ego' folder ...`, async (spinner) => {
                    const STAT = await fs.stat(EGO_FOLDER_PATH);
                    if (STAT.isDirectory()) {
                        await fs.remove(EGO_FOLDER_PATH);
                    }
                    else {
                        await fs.unlink(EGO_FOLDER_PATH);
                    }
                    spinner.text = `Existing '.ego' folder removed`;
                });
            }
            else {
                console.warn(`'.ego' folder already exists!`);
                ctx.exit(1);
            }
        }
        await util_1.withSpinnerAsync(`Creating '.ego' folder ...`, async (spinner) => {
            await fs.mkdirs(EGO_FOLDER_PATH);
            spinner.text = `'.ego' folder created`;
        });
        const STORAGE_FILE_PATH = path.resolve(path.join(EGO_FOLDER_PATH, contracts_1.STORAGE_FILE));
        await util_1.withSpinnerAsync(`Creating storage file ...`, async (spinner) => {
            await fs.writeFile(STORAGE_FILE_PATH, JSON.stringify({}), 'utf8');
            spinner.text = `Storage file created`;
        });
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Options:`);
        util_1.writeLine(` -f, --force    # Remove existing folder, if needed.`);
        util_1.writeLine();
        util_1.writeLine(`Examples:  ego init`);
        util_1.writeLine(`           ego init -f`);
    }
}
exports.EgoCommand = EgoCommand;
//# sourceMappingURL=index.js.map