"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const inquirer = require("inquirer");
const contracts_1 = require("../../contracts");
const util_1 = require("../../util");
/**
 * New command.
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = 'Starts the e.GO generator for Yeoman.';
    }
    /** @inheritdoc */
    async execute(ctx) {
        const USE_YARN = ctx.args['y'] ||
            ctx.args['yarn'] ||
            ctx.get('yarn');
        let installYeomanAction;
        let installEGOGeneratorAction;
        if (USE_YARN) {
            installYeomanAction = async () => {
                await util_1.spawnAsync('yarn', ['global', 'add', 'yo'], {
                    cwd: ctx.cwd,
                    stdio: util_1.getSTDIO(ctx),
                });
            };
            installEGOGeneratorAction = async () => {
                await util_1.spawnAsync('yarn', ['global', 'add', 'generator-ego'], {
                    cwd: ctx.cwd,
                    stdio: util_1.getSTDIO(ctx),
                });
            };
        }
        else {
            installYeomanAction = async () => {
                await util_1.spawnAsync('npm', ['install', '-g', 'yo'], {
                    cwd: ctx.cwd,
                    stdio: util_1.getSTDIO(ctx),
                });
            };
            installEGOGeneratorAction = async () => {
                await util_1.spawnAsync('npm', ['install', '-g', 'generator-ego'], {
                    cwd: ctx.cwd,
                    stdio: util_1.getSTDIO(ctx),
                });
            };
        }
        // Yeoman installed?
        if (!util_1.globalModuleExists('yo')) {
            const ANSWER = await inquirer.prompt([{
                    type: 'confirm',
                    name: 'ego_confirm',
                    message: 'Install Yeoman generator?',
                    default: true,
                }]);
            if (!ANSWER['ego_confirm']) {
                return;
            }
            await util_1.withSpinnerAsync(`Installing Yeoman generator ...`, async (spinner) => {
                await installYeomanAction();
                spinner.text = 'Yeoman generator installed';
            });
        }
        // e.GO generator installed?
        if (!util_1.globalModuleExists('generator-ego')) {
            const ANSWER = await inquirer.prompt([{
                    type: 'confirm',
                    name: 'ego_confirm',
                    message: util_1.eGO('Install e.GO generator for Yeoman?'),
                    default: true,
                }]);
            if (!ANSWER['ego_confirm']) {
                return;
            }
            await util_1.withSpinnerAsync(util_1.eGO(`Installing e.GO generator for Yeoman ...`), async (spinner) => {
                await installEGOGeneratorAction();
                spinner.text = util_1.eGO(`e.GO generator for Yeoman installed`);
            });
        }
        // run e.GO generator
        await util_1.spawnAsync('yo', ['ego'], {
            cwd: ctx.cwd,
        });
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Options:`);
        util_1.writeLine(` -v, --verbose  # Verbose output.`);
        util_1.writeLine(` -y, --yarn     # Use yarn instead.`);
        util_1.writeLine();
        util_1.writeLine(`Config:`);
        util_1.writeLine(` yarn   # Use yarn instead.`);
        util_1.writeLine();
        util_1.writeLine(`Examples:  ego new`);
        util_1.writeLine(`           ego new -y`);
    }
}
exports.EgoCommand = EgoCommand;
//# sourceMappingURL=index.js.map