"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const fs = require("fs-extra");
const path = require("path");
const contracts_1 = require("../../contracts");
const util_1 = require("../../util");
/**
 * Node-Install command.
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = "Removes the 'node_modules' subfolder and re-installs it.";
    }
    /** @inheritdoc */
    async execute(ctx) {
        const PACKAGE_JSON = path.resolve(path.join(ctx.cwd, 'package.json'));
        if (!(await util_1.exists(PACKAGE_JSON))) {
            console.warn(`'package.json' file not found!`);
            ctx.exit(1);
        }
        const USE_YARN = ctx.args['y'] ||
            ctx.args['yarn'] ||
            ctx.get('yarn');
        let installAction;
        let updateAction;
        if (USE_YARN) {
            installAction = async () => {
                await util_1.withSpinnerAsync(`Executing 'yarn install' ...`, async (spinner) => {
                    await util_1.spawnAsync('yarn', ['install'], {
                        cwd: ctx.cwd,
                        stdio: util_1.getSTDIO(ctx),
                    });
                    spinner.text = `'yarn install' executed`;
                });
            };
            updateAction = async () => {
                await util_1.withSpinnerAsync(`Executing 'yarn upgrade' ...`, async (spinner) => {
                    await util_1.spawnAsync('yarn', ['upgrade'], {
                        cwd: ctx.cwd,
                        stdio: util_1.getSTDIO(ctx),
                    });
                    spinner.text = `'yarn upgrade' executed`;
                });
            };
        }
        else {
            installAction = async () => {
                await util_1.withSpinnerAsync(`Executing 'npm install' ...`, async (spinner) => {
                    await util_1.spawnAsync('npm', ['install'], {
                        cwd: ctx.cwd,
                        stdio: util_1.getSTDIO(ctx),
                    });
                    spinner.text = `'npm install' executed`;
                });
            };
            updateAction = async () => {
                await util_1.withSpinnerAsync(`Executing 'npm update' ...`, async (spinner) => {
                    await util_1.spawnAsync('npm', ['update'], {
                        cwd: ctx.cwd,
                        stdio: util_1.getSTDIO(ctx),
                    });
                    spinner.text = `'npm update' executed`;
                });
            };
        }
        await util_1.withSpinnerAsync(`Removing 'node_modules' folder ...`, async (spinner) => {
            const NODE_MODULES_FOLDER = path.resolve(path.join(ctx.cwd, 'node_modules'));
            if (await util_1.exists(NODE_MODULES_FOLDER)) {
                const STAT = await fs.lstat(NODE_MODULES_FOLDER);
                if (STAT.isDirectory()) {
                    await fs.remove(NODE_MODULES_FOLDER);
                }
                else if (STAT.isSymbolicLink()) {
                    await fs.unlink(NODE_MODULES_FOLDER);
                }
            }
            spinner.text = `'node_modules' folder removed`;
        });
        await installAction();
        // update?
        if (ctx.args['u'] || ctx.args['update']) {
            await updateAction();
        }
        // npm audit fix?
        if (ctx.args['a'] || ctx.args['audit']) {
            await util_1.withSpinnerAsync(`Executing 'npm audit fix' ...`, async (spinner) => {
                await util_1.spawnAsync('npm', ['audit', 'fix'], {
                    cwd: ctx.cwd,
                    stdio: util_1.getSTDIO(ctx),
                });
                spinner.text = `'npm audit fix' executed`;
            });
        }
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Options:`);
        util_1.writeLine(` -a, --audit    # Runs 'npm audit fix' after successful execution.`);
        util_1.writeLine(` -u, --update   # Runs 'update' after successful execution.`);
        util_1.writeLine(` -v, --verbose  # Verbose output.`);
        util_1.writeLine(` -y, --yarn     # Use yarn instead.`);
        util_1.writeLine();
        util_1.writeLine(`Config:`);
        util_1.writeLine(` yarn   # Use yarn instead.`);
        util_1.writeLine();
        util_1.writeLine(`Examples:  ego node-install`);
        util_1.writeLine(`           ego node-install -y`);
        util_1.writeLine(`           ego node-install -a`);
        util_1.writeLine(`           ego node-install -ua`);
    }
}
exports.EgoCommand = EgoCommand;
//# sourceMappingURL=index.js.map