"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const open = require("open");
const contracts_1 = require("../../contracts");
const git_1 = require("../../git");
const util_1 = require("../../util");
const REGEX_HTTP_URL = /(https\:\/\/)([^\.]+)(\.)(visualstudio\.com)(\/)([^\/]+)(\/)(_git)(\/)([\S]+)/i;
const REGEX_SSH_URL = /([^@]+)(@)([^\:]+)(\:)([^\/]+)(\/)([^\/]+)(\/)([^\/]+)(\/)([\S]+)/i;
/**
 * Pull-Request command.
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = "Starts a pull request (Azure DevOps) for the branch of the current git repository.";
    }
    /** @inheritdoc */
    async execute(ctx) {
        const IGNORE_CHANGES = ctx.args['i'] ||
            ctx.args['ignore-changes'];
        if (!IGNORE_CHANGES) {
            if (git_1.hasGitChanges(ctx)) {
                console.warn(`Repository has uncommited changes!`);
                ctx.exit(1);
            }
        }
        // target branch
        let targetBranch = util_1.toStringSafe(ctx.args['t'])
            .trim();
        if ('' === targetBranch) {
            targetBranch = util_1.toStringSafe(ctx.args['target'])
                .trim();
        }
        if ('' === targetBranch) {
            // try get default from config
            targetBranch = util_1.toStringSafe(ctx.get('default_target_branch'))
                .trim();
        }
        if ('' === targetBranch) {
            // default
            targetBranch = 'master';
        }
        const NO_PUSH = ctx.args['np'] ||
            ctx.args['no-push'];
        // current / source branch
        let srcBranch;
        await util_1.withSpinnerAsync('Detecting current branch ...', async (spinner) => {
            srcBranch = git_1.getCurrentGitBranch({
                cwd: ctx.cwd,
                verbose: ctx.verbose,
            });
            if (srcBranch === targetBranch) {
                spinner.warn('Same branch!');
                ctx.exit(2);
            }
            spinner.text = `Branch: '${srcBranch}'`;
        });
        // loading remotes
        let remotes;
        await util_1.withSpinnerAsync('Loading remotes ...', async (spinner) => {
            remotes = git_1.getGitRemotes({
                cwd: ctx.cwd,
                verbose: ctx.verbose,
            });
            spinner.text = remotes.length + ' remote(s) found';
        });
        // get repository details
        const REMOTES_WITH_URLS = [];
        for (let i = 0; i < remotes.length; i++) {
            const R = remotes[i];
            await util_1.withSpinnerAsync(`Getting remote URL of '${R}' (${i + 1} / ${remotes.length}) ...`, async (spinner) => {
                const REMOTE_URL = git_1.getGitRemoteUrl(R, {
                    cwd: ctx.cwd,
                    verbose: ctx.verbose,
                });
                if (false === REMOTE_URL) {
                    spinner.warn(`Could not get remote URL of '${R}' (${i + 1} / ${remotes.length})!`);
                }
                else {
                    const NEW_ENTRY = {
                        name: R,
                        repository: false,
                        url: REMOTE_URL,
                    };
                    const SSH = REGEX_SSH_URL.exec(NEW_ENTRY.url);
                    if (null === SSH) {
                        const HTTP = REGEX_HTTP_URL.exec(NEW_ENTRY.url);
                        if (null !== HTTP) {
                            // HTTP URL
                            NEW_ENTRY.repository = {
                                name: decodeURIComponent(HTTP[10]).trim(),
                                organization: decodeURIComponent(HTTP[2]),
                                project: decodeURIComponent(HTTP[6]),
                            };
                        }
                    }
                    else {
                        // SSH URL
                        NEW_ENTRY.repository = {
                            name: decodeURIComponent(SSH[11]),
                            organization: decodeURIComponent(SSH[7]),
                            project: decodeURIComponent(SSH[9]),
                        };
                    }
                    if (false !== NEW_ENTRY.repository) {
                        // has enough information
                        REMOTES_WITH_URLS.push(NEW_ENTRY);
                    }
                    spinner.text = `Remote URL of '${R}' got (${i + 1} / ${remotes.length})`;
                }
            });
        }
        // push to remotes?
        if (!NO_PUSH) {
            for (let i = 0; i < REMOTES_WITH_URLS.length; i++) {
                const R = REMOTES_WITH_URLS[i];
                await util_1.withSpinnerAsync(`Pushing to '${R.name}' (${i + 1} / ${REMOTES_WITH_URLS.length}) ...`, async (spinner) => {
                    await util_1.spawnAsync('git', ['push', R.name, srcBranch], {
                        cwd: ctx.cwd,
                        stdio: util_1.getSTDIO(ctx),
                    });
                    spinner.text = `Pushed to '${R.name}' (${i + 1} / ${REMOTES_WITH_URLS.length})`;
                });
            }
        }
        // open in browser
        for (let i = 0; i < REMOTES_WITH_URLS.length; i++) {
            const R = REMOTES_WITH_URLS[i];
            const REPO_DETAILS = R.repository;
            await util_1.withSpinnerAsync(`Opening pull request in browser for '${R.name}' (${i + 1} / ${REMOTES_WITH_URLS.length}) ...`, async (spinner) => {
                const WEB_URL = `https://${encodeURIComponent(REPO_DETAILS.organization)}.visualstudio.com/${encodeURIComponent(REPO_DETAILS.project)}/_git/${encodeURIComponent(REPO_DETAILS.name)}/pullrequestcreate?sourceRef=${encodeURIComponent(srcBranch)}&targetRef=${encodeURIComponent(targetBranch)}`;
                await open(WEB_URL, {
                    wait: false,
                });
                spinner.text = `Pull request for '${R.name}' opened (${i + 1} / ${REMOTES_WITH_URLS.length})`;
            });
        }
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Options:`);
        util_1.writeLine(` -i, --ignore-changes  # Ignore uncommited local changes or not. Default: (false)`);
        util_1.writeLine(` --np, --no-push       # Indicates NOT pushing local changes to remote. Default: (false)`);
        util_1.writeLine(` --t, --target         # The name of the target branch. Default: master`);
        util_1.writeLine(` -v, --verbose         # Verbose output.`);
        util_1.writeLine();
        util_1.writeLine(`Config:`);
        util_1.writeLine(` default_target_branch   # The name of the default target branch.`);
        util_1.writeLine();
        util_1.writeLine(`Examples:   ego pull-request`);
        util_1.writeLine(`            ego pull-request --target=dev`);
        util_1.writeLine(`            ego pull-request --np --target=stage`);
    }
}
exports.EgoCommand = EgoCommand;
//# sourceMappingURL=index.js.map