"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const path = require("path");
const qrcode = require('qrcode');
const contracts_1 = require("../../contracts");
const util_1 = require("../../util");
/**
 * Qr command.
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = "Creates an image file with a QR code from a text.";
        /** @inheritdoc */
        this.syntax = 'TEXT [options]';
    }
    /** @inheritdoc */
    async execute(ctx) {
        const TEXT = util_1.toStringSafe(ctx.args['_'][0]);
        if ('' === TEXT) {
            console.warn("Please defined a text!");
            ctx.exit(1);
        }
        // margin
        let margin = parseInt(util_1.toStringSafe(ctx.args['m']).trim());
        if (isNaN(margin)) {
            margin = parseInt(util_1.toStringSafe(ctx.args['margin']).trim());
        }
        if (isNaN(margin)) {
            margin = 4;
        }
        // scale
        let scale = parseInt(util_1.toStringSafe(ctx.args['m']).trim());
        if (isNaN(scale)) {
            scale = parseInt(util_1.toStringSafe(ctx.args['scale']).trim());
        }
        if (isNaN(scale)) {
            scale = 4;
        }
        // width
        let width = parseInt(util_1.toStringSafe(ctx.args['w']).trim());
        if (isNaN(width)) {
            width = parseInt(util_1.toStringSafe(ctx.args['width']).trim());
        }
        if (isNaN(width)) {
            width = 1024;
        }
        let format = util_1.toStringSafe(ctx.args['f'])
            .toLowerCase()
            .trim();
        if ('' === format) {
            format = util_1.toStringSafe(ctx.args['format'])
                .toLowerCase()
                .trim();
        }
        let fileExt;
        switch (format) {
            case '':
            case 's':
            case 'svg':
                {
                    format = 'svg';
                    fileExt = 'svg';
                }
                break;
            case 'p':
            case 'png':
                {
                    format = 'png';
                    fileExt = 'png';
                }
                break;
            case 't':
            case 'text':
            case 'txt':
                {
                    format = 'utf8';
                    fileExt = 'txt';
                }
                break;
            default:
                {
                    console.warn(`Wrong format '${format}'!`);
                    ctx.exit(1);
                }
                break;
        }
        const QRCODE_OPTIONS = {
            margin,
            scale,
            type: format,
            width,
        };
        // search for unique filename
        let outputFile;
        await util_1.withSpinnerAsync(`Searching for output filename ...`, async (spinner) => {
            const BASE_NAME = 'qrcode';
            const FILE_EXT = '.' + fileExt;
            let fileName = BASE_NAME + FILE_EXT;
            let i = 0;
            do {
                spinner.text = `Searching for output filename ('${fileName}') ...`;
                outputFile = path.resolve(path.join(ctx.cwd, fileName));
                if (!(await util_1.exists(outputFile))) {
                    break;
                }
                ++i;
                fileName = `${BASE_NAME}-${i}${FILE_EXT}`;
            } while (true);
            spinner.text = `Found name for output file: '${fileName}'`;
        });
        // write to output file
        await util_1.withSpinnerAsync(`Creating QR code with '${TEXT}' ...`, async (spinner) => {
            await (() => {
                return new Promise((resolve, reject) => {
                    try {
                        qrcode.toFile(outputFile, TEXT, QRCODE_OPTIONS, (err) => {
                            if (err) {
                                reject(err);
                            }
                            else {
                                resolve(undefined);
                            }
                        });
                    }
                    catch (e) {
                        reject(e);
                    }
                });
            })();
            spinner.text = `Saved QR code with '${TEXT}' to '${outputFile}'`;
        });
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Options:`);
        util_1.writeLine(` --f, --format  # The output format. Default: svg`);
        util_1.writeLine(`                # Possible values: png, svg or txt`);
        util_1.writeLine(` --m, --margin  # The margin. Default: 4`);
        util_1.writeLine(` --s, --scale   # Scale factor (each factor is 1 pixel per dot). Default: 4`);
        util_1.writeLine(` --w, --width   # The width, in pixels. Default: 1024`);
        util_1.writeLine();
        util_1.writeLine(`Examples:  ego qr "https://e-go-digital.com"`);
        util_1.writeLine(`           ego qr "https://enterprisecockpit.e-go-digital.com" --format=png`);
        util_1.writeLine(`           ego qr "https://e-base.e-go-digital.com" --width=512`);
        util_1.writeLine(`           ego qr "https://github.com/egodigital/vscode-powertools" --margin=2`);
        util_1.writeLine(`           ego qr "https://github.com/egodigital/express-controllers" --scale=2`);
    }
}
exports.EgoCommand = EgoCommand;
//# sourceMappingURL=index.js.map