"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const _ = require("lodash");
const contracts_1 = require("../../contracts");
const util_1 = require("../../util");
/**
 * Rn-Run command.
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = "Runs the current React Native project.";
    }
    /** @inheritdoc */
    async execute(ctx) {
        let android = false;
        let iOS = false;
        // first get defaults
        const DEFAULT_TARGETS = util_1.toStringSafe(ctx.get('react_native_targets'))
            .split(',')
            .map(x => x.toLowerCase().trim());
        for (const DT of DEFAULT_TARGETS) {
            switch (DT) {
                case 'a':
                case 'android':
                    android = true;
                    break;
                case 'i':
                case 'ios':
                    iOS = true;
                    break;
            }
        }
        if ((!_.isNil(ctx.args['a']) || !_.isNil(ctx.args['android'])) &&
            (!_.isNil(ctx.args['i']) || !_.isNil(ctx.args['ios']))) {
            // use CLI arguments
            android = ctx.args['a'] ||
                ctx.args['android'];
            iOS = ctx.args['i'] ||
                ctx.args['ios'];
        }
        if (!android && !iOS) {
            // default by operation system
            switch (process.platform) {
                case 'darwin':
                    iOS = true;
                    break;
                default:
                    android = true;
                    break;
            }
        }
        if (iOS) {
            await util_1.withSpinnerAsync(`Opening app on iOS ...`, async (spinner) => {
                try {
                    await util_1.spawnAsync('npx', ['react-native', 'run-ios'], {
                        cwd: ctx.cwd,
                        stdio: util_1.getSTDIO(ctx),
                    });
                    spinner.text = `App opened on iOS`;
                }
                catch (error) {
                    spinner.fail(`Could not open app on iOS. (Error: ${error.message}`);
                }
            });
        }
        if (android) {
            await util_1.withSpinnerAsync(`Opening app on Android ...`, async (spinner) => {
                try {
                    await util_1.spawnAsync('npx', ['react-native', 'run-android'], {
                        cwd: ctx.cwd,
                        stdio: util_1.getSTDIO(ctx),
                    });
                    spinner.text = `App opened on Android`;
                }
                catch (error) {
                    spinner.fail(`Could not open app on Android. (Error: ${error.message}`);
                }
            });
        }
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Options:`);
        util_1.writeLine(` -a, --android  # Opens app on Android.`);
        util_1.writeLine(` -i, --ios      # Opens app on iOS.`);
        util_1.writeLine(` -v, --verbose  # Verbose output.`);
        util_1.writeLine();
        util_1.writeLine(`Config:`);
        util_1.writeLine(` react_native_targets   # A comma separated list of default targets.`);
        util_1.writeLine(`                        # Example: ios,android`);
        util_1.writeLine();
        util_1.writeLine(`Examples:  ego rn-run --ios`);
        util_1.writeLine(`           ego rn-run -ia`);
    }
}
exports.EgoCommand = EgoCommand;
//# sourceMappingURL=index.js.map