"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const web_api_1 = require("@slack/web-api");
const contracts_1 = require("../../contracts");
const util_1 = require("../../util");
/**
 * Slack-Post command.
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = 'Posts a message to one or more Slack channels.';
        /** @inheritdoc */
        this.syntax = 'MESSAGE [options]';
    }
    /** @inheritdoc */
    async execute(ctx) {
        const MESSAGE = util_1.toStringSafe(ctx.args['_'][0])
            .trim();
        if ('' === MESSAGE) {
            console.warn(`Please define a message!`);
            ctx.exit(1);
        }
        let slackToken = util_1.toStringSafe(ctx.args['t'])
            .trim();
        if ('' === slackToken) {
            slackToken = util_1.toStringSafe(ctx.args['token'])
                .trim();
            if ('' === slackToken) {
                slackToken = util_1.toStringSafe(ctx.get('slack_token'))
                    .trim();
            }
        }
        if ('' === slackToken) {
            console.warn(`Please setup ${util_1.colorize('slack_token')} config value, by executing ${util_1.colorize('ego set slack_token "<MY-SLACK-TOKEN>"')}`);
            ctx.exit(2);
        }
        let channelList = util_1.toStringSafe(ctx.args['c'])
            .trim();
        if ('' === channelList) {
            channelList = util_1.toStringSafe(ctx.args['channels'])
                .trim();
        }
        const CHANNELS = channelList.split(',').map(c => {
            return c.trim();
        }).filter(c => '' !== c);
        const SLACK_CLIENT = new web_api_1.WebClient(slackToken);
        for (const C of CHANNELS) {
            await util_1.withSpinnerAsync(`Posting message to Slack channel '${C}' ...`, async (spinner) => {
                try {
                    await SLACK_CLIENT.chat.postMessage({
                        channel: C,
                        text: MESSAGE,
                    });
                    spinner.text = `Posted message to Slack channel '${C}'`;
                }
                catch (e) {
                    spinner.fail(`Posting message to Slack '${C}' failed: '${util_1.toStringSafe(e)}'`);
                }
            });
        }
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Options:`);
        util_1.writeLine(` --c, --channels  # One or more channels to post to, separated by commas.`);
        util_1.writeLine(` --t, --token     # The custom token of the (bot) user for the Slack Web API.`);
        util_1.writeLine();
        util_1.writeLine(`Config:`);
        util_1.writeLine(` slack_token  # The token of the (bot) user for the Slack Web API.`);
        util_1.writeLine();
        util_1.writeLine(`Example:  ego slack-post "Hello, e.GO" --channels=ABCDEFGHI,JKLMNOPQR`);
    }
}
exports.EgoCommand = EgoCommand;
//# sourceMappingURL=index.js.map