"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const fs = require("fs-extra");
const path = require("path");
const contracts_1 = require("../../contracts");
const util_1 = require("../../util");
/**
 * Ssl-New command.
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = 'Creates a new self-signed SSL certificate.';
    }
    /** @inheritdoc */
    async execute(ctx) {
        const OVERWRITE = ctx.args['o'] ||
            ctx.args['overwrite'];
        const EGO_FOLDER = util_1.getEGOFolder();
        let i = 0;
        let certFile;
        let keyFile;
        const UPDATE_FILES = () => {
            certFile = path.resolve(path.join(EGO_FOLDER, `server${0 !== i ? ('-' + i) : ''}.cert`));
            keyFile = path.resolve(path.join(EGO_FOLDER, `server${0 !== i ? ('-' + i) : ''}.key`));
            ++i;
        };
        await util_1.withSpinnerAsync(`Defining output files ...`, async (spinner) => {
            do {
                UPDATE_FILES();
                if (OVERWRITE) {
                    break; // do not check for duplicate files
                }
                if (!(await util_1.exists(certFile)) && !(await util_1.exists(keyFile))) {
                    break;
                }
            } while (true);
            spinner.text = `Output files defined`;
        });
        if (OVERWRITE) {
            if (await util_1.exists(certFile)) {
                await fs.unlink(certFile);
            }
            if (await util_1.exists(keyFile)) {
                await fs.unlink(keyFile);
            }
        }
        // openssl req -nodes -newkey rsa:4096 -x509 -days 3650 -sha256 -keyout <keyFile> -out <certFile>
        const OPENSSL_ARGS = [
            'req', '-nodes', '-newkey', 'rsa:4096', '-x509', '-days', '3650', '-sha256',
            '-keyout', keyFile,
            '-out', certFile,
        ];
        if (ctx.verbose) {
            OPENSSL_ARGS.push('--verbose');
        }
        await util_1.spawnAsync('openssl', OPENSSL_ARGS);
        util_1.writeLine();
        util_1.writeLine(`CERT file: ${certFile}`);
        util_1.writeLine(`KEY file: ${keyFile}`);
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Options:`);
        util_1.writeLine(` -o, --overwrite  # Overwrite existing files or not.`);
        util_1.writeLine(` -v, --verbose    # Verbose output.`);
        util_1.writeLine();
        util_1.writeLine(`Examples:  ego ssl-new`);
        util_1.writeLine(`           ego ssl-new -o`);
    }
}
exports.EgoCommand = EgoCommand;
//# sourceMappingURL=index.js.map