"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EgoCommand = void 0;
const cliHighlight = require("cli-highlight");
const chokidar = require("chokidar");
const fs = require("fs-extra");
const path = require("path");
const p_queue_1 = require("p-queue");
const contracts_1 = require("../../contracts");
const os_1 = require("os");
const scripts_1 = require("../../scripts");
const util_1 = require("../../util");
/**
 * Watch command.
 */
class EgoCommand extends contracts_1.CommandBase {
    constructor() {
        super(...arguments);
        /** @inheritdoc */
        this.description = "Runs one or more scripts for file changes.";
        /** @inheritdoc */
        this.syntax = '[WATCHER_FILE+]';
    }
    /** @inheritdoc */
    async execute(ctx) {
        const ARGS = util_1.asArray(ctx.args['_'])
            .map(x => util_1.toStringSafe(x))
            .filter(x => '' !== x);
        if (ARGS.length < 1) {
            console.warn('Define at least one script file, which should be executed!');
            ctx.exit(1);
        }
        const APPEND_SCRIPT_EXTENSION = (filePath) => {
            if (!filePath.endsWith(scripts_1.getShellScriptExtension())) {
                if (!filePath.endsWith('.js')) {
                    filePath += '.js';
                }
            }
            return filePath;
        };
        const WATCH_ACTIONS = [];
        ARGS.forEach(sf => {
            let scriptFile = sf;
            if (!path.isAbsolute(scriptFile)) {
                let newScriptPath = path.join(ctx.cwd, scriptFile);
                newScriptPath = APPEND_SCRIPT_EXTENSION(newScriptPath);
                if (!fs.existsSync(newScriptPath)) {
                    // try from '.ego' folder
                    newScriptPath = path.join(util_1.getEGOFolder(), scriptFile);
                }
                scriptFile = newScriptPath;
            }
            scriptFile = require.resolve(scriptFile);
            scriptFile = APPEND_SCRIPT_EXTENSION(scriptFile);
            if (scriptFile.endsWith(scripts_1.getShellScriptExtension())) {
                WATCH_ACTIONS.push(async (ev, p) => {
                    await this._executeShellScript(ctx, scriptFile, ev, p);
                });
            }
            else {
                WATCH_ACTIONS.push(async (ev, p) => {
                    await this._executeScriptFile(ctx, scriptFile, ev, p);
                });
            }
        });
        const WATCHER = chokidar.watch(ctx.cwd, {
            cwd: ctx.cwd,
        });
        let isReady = false;
        const QUEUE = new p_queue_1.default({
            autoStart: true,
            concurrency: 1,
        });
        const INVOKE_EVENT = (ev, p) => {
            if (!isReady) {
                return;
            }
            QUEUE.add(async () => {
                for (const WA of WATCH_ACTIONS) {
                    try {
                        await WA(ev, p);
                    }
                    catch (e) {
                        util_1.writeErrLine(e);
                    }
                }
            }).catch((err) => {
                util_1.writeErrLine(err);
            });
        };
        WATCHER.on('error', err => {
            util_1.writeErrLine(err);
        }).on('add', p => {
            INVOKE_EVENT('file:add', p);
        }).on('change', p => {
            INVOKE_EVENT('file:change', p);
        }).on('unlink', p => {
            INVOKE_EVENT('file:unlink', p);
        }).on('addDir', p => {
            INVOKE_EVENT('dir:add', p);
        }).on('unlinkDir', p => {
            INVOKE_EVENT('dir:unlink', p);
        }).on('ready', () => {
            isReady = true;
        });
        await util_1.waitForEnter(`Press <ENTER> to stop ...${os_1.EOL}`);
        WATCHER.close();
        ctx.exit();
    }
    /** @inheritdoc */
    async showHelp() {
        util_1.writeLine(`Examples:  ego watch my-watcher.js`);
        util_1.writeLine(`           ego watch my-watcher`);
        util_1.writeLine();
        util_1.writeLine(`Relative paths will be mapped to the current working directory or the '.ego' subfolder inside the user's home directory.`);
        util_1.writeLine();
        util_1.writeLine(`It is also possible to run shell scripts (with .sh or .cmd extension).`);
        util_1.writeLine();
        util_1.writeLine();
        util_1.writeLine(`Example script:`);
        util_1.writeLine(cliHighlight.highlight(`
    exports.execute = async (path, event, context) => {
        // context  =>  s. https://egodigital.github.io/ego-cli/interfaces/_contracts_.commandexecutecontext.html

        // context.args           =>  List of command line arguments, s. https://www.npmjs.com/package/minimist
        // context.cwd            =>  The full path of the current working directory
        // context.getFullPath()  =>  Returns the full version of a path, based on the value of 'cwd'
        // context.package        =>  The 'package.json' file of the e.GO CLI
        // context.queue          =>  A queue, that only executes 1 action at the same time, s. https://www.npmjs.com/package/p-queue
        // context.require()      =>  Allows to include a NPM module of the e.GO CLI
        // context.values         =>  A key/value pair storage, that is available while the execution
        // context.verbose        =>  Indicates, if script should output additional information or not

        // docker utils, s. https://egodigital.github.io/ego-cli/modules/_docker_.html
        const docker = context.require('./docker');
        // git utils, s. https://egodigital.github.io/ego-cli/modules/_git_.html
        const git = context.require('./git');
        // common app utils, s. https://egodigital.github.io/ego-cli/modules/_util_.html
        const util = context.require('./util');

        util.writeLine(\`'\${path}' raised the '\${event}' event.\`);
    };
`, {
            language: 'javascript',
        }));
    }
    async _executeShellScript(ctx, scriptFile, ev, p) {
        await util_1.spawnAsync(scriptFile, [p, ev], {
            cwd: ctx.cwd,
        });
    }
    async _executeScriptFile(ctx, scriptFile, ev, p) {
        delete require.cache[scriptFile];
        const SCRIPT_MODULE = require(scriptFile);
        if (SCRIPT_MODULE) {
            const EXECUTE = SCRIPT_MODULE.execute;
            if (EXECUTE) {
                await Promise.resolve(EXECUTE.apply(SCRIPT_MODULE, [p, ev, ctx]));
            }
        }
    }
}
exports.EgoCommand = EgoCommand;
//# sourceMappingURL=index.js.map