"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.hasGitChanges = exports.getGitRemoteUrl = exports.getGitRemotes = exports.getGitBranches = exports.getCurrentGitBranch = void 0;
const _ = require("lodash");
const util_1 = require("./util");
/**
 * Gets the current git branch.
 *
 * @param {GetCurrentGitBranchOptions} [opts] Custom options.
 *
 * @return {string} The name of the current branch.
 */
function getCurrentGitBranch(opts) {
    return util_1.toStringSafe(util_1.asArray(util_1.spawn('git', ['rev-parse', '--abbrev-ref', 'HEAD'], {
        cwd: util_1.getCWD(opts),
        stdio: null,
    }).output).join('')).trim();
}
exports.getCurrentGitBranch = getCurrentGitBranch;
/**
 * Lists all branches of a git repository.
 *
 * @param {GetGitBranchesOptions} [opts] Custom options.
 *
 * @return {string[]} The list of branches.
 */
function getGitBranches(opts) {
    if (_.isNil(opts)) {
        opts = {};
    }
    return util_1.asArray(util_1.spawn('git', ['branch'], {
        cwd: util_1.getCWD(opts),
        stdio: null,
    }).output).map((x) => {
        return util_1.toStringSafe(x)
            .split('\n');
    }).reduce((x, y) => {
        return x.concat(y);
    }, []).sort((x, y) => {
        return util_1.compareValuesBy(x, y, (i) => {
            return i.trim()
                .startsWith('* ') ? 0 : 1;
        });
    }).map(b => {
        b = b.trim();
        if (b.startsWith('*')) {
            b = b.substr(1)
                .trim();
        }
        return b;
    }).filter(b => '' !== b);
}
exports.getGitBranches = getGitBranches;
/**
 * Lists all remotes of a git repository.
 *
 * @param {GetGitRemotesOptions} [opts] Custom options.
 *
 * @return {string[]} The list of remotes.
 */
function getGitRemotes(opts) {
    if (_.isNil(opts)) {
        opts = {};
    }
    return util_1.asArray(util_1.spawn('git', ['remote'], {
        cwd: util_1.getCWD(opts),
        stdio: null,
    }).output).map((x) => {
        return util_1.toStringSafe(x)
            .split('\n');
    }).reduce((x, y) => {
        return x.concat(y);
    }, []).sort((x, y) => {
        return util_1.compareValuesBy(x, y, (i) => {
            return 'origin' === i.trim()
                ? 0 : 1;
        });
    }).map(r => {
        return r.trim();
    }).filter(r => '' !== r);
}
exports.getGitRemotes = getGitRemotes;
/**
 * Tries to return the remote URL of a git repository.
 *
 * @param {string} [remote] The name of the remote. Default: 'origin'.
 * @param {GetGitRemoteUrlOptions} [opts] Custom options.
 *
 * @return {string|false} The remote URL or (false) if not possible.
 */
function getGitRemoteUrl(remote, opts) {
    remote = util_1.toStringSafe(remote)
        .trim();
    if ('' === remote) {
        remote = 'origin';
    }
    if (_.isNil(opts)) {
        opts = {};
    }
    const REMOTE_URL = util_1.asArray(util_1.spawn('git', ['remote', 'get-url', remote], {
        stdio: null
    }).output).map(x => util_1.toStringSafe(x).trim())
        .filter(x => '' !== x)
        .join('')
        .trim();
    if ('' === REMOTE_URL) {
        return false;
    }
    return REMOTE_URL;
}
exports.getGitRemoteUrl = getGitRemoteUrl;
/**
 * Checks if a git repository has uncommited changes or not.
 *
 * @param {HasGitChangesOptions} [opts] Custom options.
 *
 * @return {boolean} Has uncomitted changes or not.
 */
function hasGitChanges(opts) {
    if (_.isNil(opts)) {
        opts = {};
    }
    return util_1.asArray(util_1.spawn('git', ['status', '-s'], {
        cwd: util_1.getCWD(opts),
        stdio: null,
    }).output).map((x) => {
        return util_1.toStringSafe(x)
            .split('\n');
    }).reduce((x, y) => {
        return x.concat(y);
    }, []).filter(x => '' !== x.trim())
        .length > 0;
}
exports.hasGitChanges = hasGitChanges;
//# sourceMappingURL=git.js.map