"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.suggestCommand = exports.showHelpHeader = exports.showHelp = void 0;
const _ = require("lodash");
const chalk = require('chalk');
const fs = require("fs-extra");
const levenshtein = require('js-levenshtein');
const path = require("path");
const util_1 = require("./util");
/**
 * Shows the common help screen.
 *
 * @param {PackageJSON} app The app information.
 */
async function showHelp(app) {
    showHelpHeader(app);
    util_1.writeLine(`Syntax:    ego COMMAND [options]`);
    util_1.writeLine();
    util_1.writeLine(`General options:`);
    util_1.writeLine(` -h, --help     # Prints this info and help screen.`);
    util_1.writeLine(` -v, --version  # Prints the version of that app.`);
    util_1.writeLine();
    util_1.writeLine(`Examples:    ego -v`);
    util_1.writeLine(`             ego new`);
    util_1.writeLine(`             ego help git-sync`);
    util_1.writeLine();
    util_1.writeLine(`Available commands:`);
    {
        const COMMANDS = [];
        const COMMANDS_DIR = path.resolve(path.join(__dirname, 'commands'));
        for (const ITEM of await fs.readdir(COMMANDS_DIR)) {
            const CMD_MODULE_FILE = require.resolve(path.join(COMMANDS_DIR, ITEM, 'index.js'));
            const STAT = await fs.stat(CMD_MODULE_FILE);
            if (STAT.isFile()) {
                const COMMAND_MODULE = require(CMD_MODULE_FILE);
                const COMMAND_CLASS = COMMAND_MODULE.EgoCommand;
                if (_.isNil(COMMAND_CLASS)) {
                    continue;
                }
                COMMANDS.push({
                    file: CMD_MODULE_FILE,
                    name: ITEM,
                    object: new COMMAND_CLASS(),
                });
            }
        }
        // sort commands by name
        const COMMANDS_SORTED = COMMANDS.sort((x, y) => {
            return util_1.compareValuesBy(x, y, (i) => {
                return i.name;
            });
        });
        let maxCommandNameLength = 0;
        COMMANDS_SORTED.forEach(c => {
            maxCommandNameLength = Math.max(maxCommandNameLength, c.name.length);
        });
        for (const CMD of COMMANDS_SORTED) {
            util_1.writeLine(` ${CMD.name}${' '.repeat(maxCommandNameLength - CMD.name.length)}  # ${util_1.eGO(CMD.object.description)}`);
        }
    }
}
exports.showHelp = showHelp;
/**
 * Shows the common app header for help screen.
 *
 * @param {PackageJSON} app The app information.
 */
function showHelpHeader(app) {
    util_1.writeLine(`${util_1.eGO(app.displayName)} (${app.name}) - Version ${app.version}`);
    util_1.writeLine(`by ${util_1.eGO() + chalk.reset(' Digital GmbH <') + chalk.white('https://e-go-digital.com') + chalk.reset('>')}`);
    util_1.writeLine();
}
exports.showHelpHeader = showHelpHeader;
/**
 * Suggest a command.
 *
 * @param {any} cmd The name of the entered command.
 */
async function suggestCommand(cmd) {
    cmd = util_1.toStringSafe(cmd)
        .toLowerCase()
        .trim();
    const COMMANDS_DIR = path.resolve(path.join(__dirname, 'commands'));
    const COMMANDS = [];
    for (const ITEM of await fs.readdir(COMMANDS_DIR)) {
        const CMD_MODULE_FILE = require.resolve(path.join(COMMANDS_DIR, ITEM, 'index.js'));
        const STAT = await fs.stat(CMD_MODULE_FILE);
        if (STAT.isFile()) {
            const COMMAND_MODULE = require(CMD_MODULE_FILE);
            const COMMAND_CLASS = COMMAND_MODULE.EgoCommand;
            if (!_.isNil(COMMAND_CLASS)) {
                COMMANDS.push(ITEM);
            }
        }
    }
    const NEAREST_CMD = COMMANDS.map(c => {
        return {
            distance: levenshtein(cmd, c),
            name: c,
        };
    }).sort((x, y) => util_1.compareValuesBy(x, y, i => i.distance))[0];
    console.warn(`❓❓❓ Command ${util_1.colorize(cmd)} not found! Did you mean ${util_1.colorize(NEAREST_CMD.name)}❓❓❓`);
}
exports.suggestCommand = suggestCommand;
//# sourceMappingURL=help.js.map