"use strict";
/**
 * This file is part of the ego-cli distribution.
 * Copyright (c) e.GO Digital GmbH, Aachen, Germany (https://www.e-go-digital.com/)
 *
 * ego-cli is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * ego-cli is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeLine = exports.writeErrLine = exports.writeErr = exports.write = exports.withSpinnerAsync = exports.withSpinner = exports.waitForEnter = exports.toStringSafe = exports.toBufferSafe = exports.toBooleanSafe = exports.spawnAsync = exports.spawn = exports.sortObjectByKeys = exports.loadResourceAsync = exports.loadResource = exports.loadEJSAsync = exports.loadEJS = exports.globalModuleExists = exports.getSTDIO = exports.getResourcePath = exports.getMimeType = exports.getEGOFolder = exports.getCWD = exports.exists = exports.eGO = exports.compareValuesBy = exports.colorize = exports.async = exports.asArray = void 0;
const _ = require("lodash");
const chalk = require('chalk');
const child_process = require("child_process");
const deepMerge = require("deepmerge");
const ejs = require("ejs");
const fs = require("fs-extra");
const globalDirs = require("global-dirs");
const mimeTypes = require("mime-types");
const ora = require("ora");
const os = require("os");
const path = require("path");
const readline = require("readline");
const util = require("util");
/**
 * Keeps sure a value is an array.
 *
 * @param {T|T[]} val The input value.
 * @param {boolean} [noEmpty] Remove values with are (null) or (undefined).
 *
 * @return {T[]} The output value.
 */
function asArray(val, noEmpty = true) {
    if (!Array.isArray(val)) {
        val = [val];
    }
    return val.filter(x => {
        if (noEmpty) {
            return !_.isNil(x);
        }
        return true;
    });
}
exports.asArray = asArray;
/**
 * Runs an action async.
 *
 * @param {Function} action The action to invoke.
 *
 * @return {Promise<TResult>} The promise with the resumt of the action.
 */
function async(action) {
    return new Promise((resolve, reject) => {
        try {
            process.nextTick(() => {
                try {
                    resolve(action());
                }
                catch (e) {
                    reject(e);
                }
            });
        }
        catch (e) {
            reject(e);
        }
    });
}
exports.async = async;
/**
 * Colorizes a string for console output.
 *
 * @param {any} str The value to colorize.
 * @param {any} color The color.
 * @param {boolean} [reset] Do a reset at the end or not.
 *
 * @return {string} The colorized string.
 */
function colorize(str, color = 'white', reset = true) {
    color = toStringSafe(color)
        .trim();
    return `${chalk[color](toStringSafe(str))}${reset ? chalk.reset() : ''}`;
}
exports.colorize = colorize;
/**
 * Compares two values for sorting, by using a selector.
 *
 * @param {T} x The first value.
 * @param {T} y The second value.
 * @param {Function} selector The function, that selects the value to compare.
 *
 * @return {number} The soirt value.
 */
function compareValuesBy(x, y, selector) {
    const VAL_X = selector(x);
    const VAL_Y = selector(y);
    if (VAL_X !== VAL_Y) {
        if (VAL_X < VAL_Y) {
            return -1;
        }
        if (VAL_X > VAL_Y) {
            return 1;
        }
    }
    return 0;
}
exports.compareValuesBy = compareValuesBy;
/**
 * Handles a value as string and adds colors to each 'e.GO' expression inside it for console output.
 *
 * @param {any} [val] The (optional) input value.
 *
 * @return {string} The output value.
 */
function eGO(val = 'e.GO') {
    return toStringSafe(val).split("e.GO").join(`${chalk.reset() + chalk.blueBright('e') + chalk.white('.') + chalk.blueBright('GO') + chalk.reset()}`);
}
exports.eGO = eGO;
/**
 * Checks if a path exists.
 *
 * @param {fs.PathLike} p The path to check.
 *
 * @return {Promise<boolean>} The promise, that indicates, if path exists or not.
 */
function exists(p) {
    return new Promise((resolve, reject) => {
        try {
            fs.exists(p, (doesExist) => {
                resolve(doesExist);
            });
        }
        catch (e) {
            reject(e);
        }
    });
}
exports.exists = exists;
/**
 * Extracts the current working directory from an object.
 *
 * @param {WithCWD} obj The object with the data.
 *
 * @return {string} The extracted data.
 */
function getCWD(obj) {
    if (_.isNil(obj)) {
        obj = {};
    }
    let cwd = toStringSafe(obj.cwd);
    if ('' === cwd.trim()) {
        cwd = process.cwd();
    }
    if (!path.isAbsolute(cwd)) {
        cwd = path.join(process.cwd(), cwd);
    }
    return path.resolve(cwd);
}
exports.getCWD = getCWD;
/**
 * Returns the possible '.ego' folder inside the user's home directory.
 *
 * @param {boolean} [create] Create folder, if it does not exist.
 *
 * @return {string} The full path of the '.ego' folder.
 */
function getEGOFolder(create = true) {
    const FOLDER_PATH = path.resolve(path.join(os.homedir(), '.ego'));
    if (create) {
        if (!fs.existsSync(FOLDER_PATH)) {
            fs.mkdirsSync(FOLDER_PATH);
        }
    }
    return FOLDER_PATH;
}
exports.getEGOFolder = getEGOFolder;
/**
 * Returns the MIME type by file path.
 *
 * @param {any} p The (file) path.
 *
 * @return {string} The mime type.
 */
function getMimeType(p) {
    p = toStringSafe(p)
        .trim();
    switch (path.extname(p)) {
        case '.js':
            return 'text/javascript';
        case '.ts':
            return 'text/plain';
    }
    const MIME_TYPE = mimeTypes.lookup(p);
    return false !== MIME_TYPE ?
        MIME_TYPE : 'application/octet-stream';
}
exports.getMimeType = getMimeType;
/**
 * Returns the full path inside the 'res' folder.
 *
 * @param {any} p The relative path.
 *
 * @return {string} The full path.
 */
function getResourcePath(p) {
    p = toStringSafe(p)
        .trim();
    return path.resolve(path.join(__dirname, 'res', p));
}
exports.getResourcePath = getResourcePath;
/**
 * Returns the value for an spawn.stdio property.
 *
 * @param {WithVerbose} obj The object with a verbose flag.
 *
 * @return {child_process.StdioOptions} The value for the property.
 */
function getSTDIO(obj) {
    if (_.isNil(obj)) {
        obj = {};
    }
    return obj.verbose ?
        'inherit' : null;
}
exports.getSTDIO = getSTDIO;
/**
 * Checks if a global module exists.
 *
 * @param {string} moduleId The ID of the module.
 *
 * @return {boolean} Module exists or not.
 */
function globalModuleExists(moduleId) {
    moduleId = toStringSafe(moduleId)
        .trim();
    try {
        const MODULE_DIR = path.resolve(path.join(globalDirs.npm.packages, moduleId));
        if (!(fs.existsSync(MODULE_DIR))) {
            return false; // module directory not found
        }
        if (!fs.statSync(MODULE_DIR).isDirectory()) {
            return false; // module path is no directory
        }
        const PACKAGE_JSON = path.resolve(path.join(MODULE_DIR, 'package.json'));
        if (!(fs.existsSync(PACKAGE_JSON))) {
            return false; // package.json not found
        }
        return fs.statSync(PACKAGE_JSON)
            .isFile(); // is package.json path a directory?
    }
    catch (_a) {
        return false;
    }
}
exports.globalModuleExists = globalModuleExists;
/**
 * Loads an EJS template from 'res' folder.
 *
 * @param {any} p The relative path inside 'res/templates' without '.ejs' extension.
 * @param {ejs.Data} [data] The optional data.
 *
 * @return {string} The rendered template.
 */
function loadEJS(p, data) {
    return ejs.render(loadResource('templates/' + toStringSafe(p).trim() + '.ejs')
        .toString('utf8'), data);
}
exports.loadEJS = loadEJS;
/**
 * Loads an EJS template from 'res' folder.
 *
 * @param {any} p The relative path inside 'res/templates' without '.ejs' extension.
 * @param {ejs.Data} [data] The optional data.
 *
 * @return {Promise<string>} The promise with the rendered template.
 */
async function loadEJSAsync(p, data) {
    return ejs.render((await loadResourceAsync('templates/' + toStringSafe(p).trim() + '.ejs'))
        .toString('utf8'), data);
}
exports.loadEJSAsync = loadEJSAsync;
/**
 * Loads a file from 'res' folder.
 *
 * @param {æny} p The relative path of the resource.
 *
 * @return {Buffer} The loaded data.
 */
function loadResource(p) {
    return fs.readFileSync(getResourcePath(p));
}
exports.loadResource = loadResource;
/**
 * Loads a file from 'res' folder.
 *
 * @param {æny} p The relative path of the resource.
 *
 * @return {Promise<Buffer>} The promise with the loaded data.
 */
async function loadResourceAsync(p) {
    return await fs.readFile(getResourcePath(p));
}
exports.loadResourceAsync = loadResourceAsync;
/**
 * Creates a clone of an object and sorts the keys.
 *
 * @param {any} obj The input object.
 *
 * @return {any} The output object.
 */
function sortObjectByKeys(obj) {
    if (_.isNil(obj)) {
        return obj;
    }
    const CLONED_OBJ = {};
    for (const KEY of Object.keys(obj).sort((x, y) => {
        return compareValuesBy(x, y, i => {
            return toStringSafe(i)
                .toLowerCase()
                .trim();
        });
    })) {
        let value = obj[KEY];
        if (_.isFunction(value)) {
            value = value.bind(CLONED_OBJ);
        }
        CLONED_OBJ[KEY] = value;
    }
    return CLONED_OBJ;
}
exports.sortObjectByKeys = sortObjectByKeys;
/**
 * Spawns a new process.
 *
 * @param {any} command The command.
 * @param {any[]} [args] The (optional) arguments for the execution.
 * @param {SpawnOptions} [opts] Custom options.
 *
 * @return {child_process.SpawnSyncReturns<Buffer>} The result of the execution.
 */
function spawn(command, args, opts) {
    command = toStringSafe(command);
    if (_.isNil(args)) {
        args = [];
    }
    opts = deepMerge({
        cwd: process.cwd(),
        stdio: 'inherit',
        env: process.env,
        encoding: 'utf8',
    }, opts || {});
    const RESULT = child_process.spawnSync(command, asArray(args, false).map(a => toStringSafe(a)), opts);
    if (!_.isNil(RESULT.error)) {
        throw RESULT.error;
    }
    if (0 !== RESULT.status) {
        throw new Error(`'${command}' exited with code '${RESULT.status}'`);
    }
    return RESULT;
}
exports.spawn = spawn;
/**
 * Spawns a new process (async).
 *
 * @param {any} command The command.
 * @param {any[]} [args] The (optional) arguments for the execution.
 * @param {SpawnAsyncOptions} [opts] Custom options.
 *
 * @return {Promise<SpawnAsyncResult>} The promise with the result of the execution.
 */
function spawnAsync(command, args, opts) {
    return new Promise((resolve, reject) => {
        try {
            command = toStringSafe(command);
            if (_.isNil(args)) {
                args = [];
            }
            opts = deepMerge({
                cwd: process.cwd(),
                env: process.env,
                stdio: 'inherit',
            }, opts || {});
            const PS = child_process.spawn(command, args, opts);
            PS.once('error', (err) => {
                reject(err);
            });
            let stdout;
            if (!_.isNil(PS.stderr)) {
                stdout = Buffer.alloc(0);
                PS.stdout.on('data', (data) => {
                    try {
                        stdout = Buffer.concat([
                            stdout, toBufferSafe(data)
                        ]);
                    }
                    catch (e) {
                        reject(e);
                    }
                });
            }
            let stderr = null;
            if (!_.isNil(PS.stderr)) {
                stderr = Buffer.alloc(0);
                PS.stderr.on('data', (data) => {
                    try {
                        stderr = Buffer.concat([
                            stderr, toBufferSafe(data)
                        ]);
                    }
                    catch (e) {
                        reject(e);
                    }
                });
            }
            PS.once('close', (code) => {
                if (0 === code) {
                    resolve({
                        status: code,
                        stderr,
                        stdout,
                    });
                }
                else {
                    reject(new Error(`'${command}' exited with code '${code}'`));
                }
            });
        }
        catch (e) {
            reject(e);
        }
    });
}
exports.spawnAsync = spawnAsync;
/**
 * Returns a value as boolean.
 *
 * @param {any} val The input value.
 * @param {any} [defaultValue] The custom default value, if input value is (null) or (undefined).
 *
 * @return {boolean} The input value as boolean.
 */
function toBooleanSafe(val, defaultValue) {
    if (_.isBoolean(val)) {
        return val;
    }
    if (_.isNil(defaultValue)) {
        return !!defaultValue;
    }
    return !!val;
}
exports.toBooleanSafe = toBooleanSafe;
/**
 * Converts data to a buffer (if needed).
 *
 * @param {any} data The input data.
 *
 * @return {Buffer} The output data.
 */
function toBufferSafe(data) {
    if (Buffer.isBuffer(data)) {
        return data;
    }
    if (_.isNil(data)) {
        return Buffer.alloc(0);
    }
    return new Buffer(toStringSafe(data), 'utf8');
}
exports.toBufferSafe = toBufferSafe;
/**
 * Converts a value to a string (if needed).
 *
 * @param {any} val The input value.
 *
 * @return {string} The output value.
 */
function toStringSafe(val) {
    if (_.isString(val)) {
        return val;
    }
    if (_.isNil(val)) {
        return '';
    }
    if (val instanceof Error) {
        return `[${val.name}] '${val.message}'`;
    }
    if (_.isFunction(val['toString'])) {
        return String(val.toString());
    }
    return util.inspect(val);
}
exports.toStringSafe = toStringSafe;
/**
 * Waits for ENTER key (in the console).
 *
 * @param {any} [text] The custom prompt text.
 */
function waitForEnter(text = '') {
    text = toStringSafe(text);
    return new Promise((resolve, reject) => {
        try {
            const INPUT = readline.createInterface({
                input: process.stdin,
                output: process.stdout,
            });
            INPUT.question(text, () => {
                INPUT.close();
                resolve();
            });
        }
        catch (e) {
            reject(e);
        }
    });
}
exports.waitForEnter = waitForEnter;
/**
 * Executes an action for a spinner.
 *
 * @param {any} text The initial text.
 * @param {Function} [action] The action to invoke.
 *
 * @return {TResult} The result of the action.
 */
function withSpinner(text, action) {
    text = toStringSafe(text);
    const SPINNER = ora(text);
    SPINNER.start();
    try {
        let result;
        if (action) {
            result = action(SPINNER);
        }
        SPINNER.succeed();
        return result;
    }
    catch (e) {
        SPINNER.fail(SPINNER.text + ' => ' + toStringSafe(e));
        throw e;
    }
    finally {
        SPINNER.stop();
    }
}
exports.withSpinner = withSpinner;
/**
 * Executes an action for a spinner (async).
 *
 * @param {any} text The initial text.
 * @param {Function} [action] The action to invoke.
 *
 * @return {Promise<TResult>} The promise with the result of the action.
 */
async function withSpinnerAsync(text, action) {
    text = toStringSafe(text);
    const SPINNER = ora(text);
    await async(() => SPINNER.start());
    try {
        let result;
        if (action) {
            result = await action(SPINNER);
        }
        await async(() => SPINNER.succeed());
        return result;
    }
    catch (e) {
        await async(() => SPINNER.fail(SPINNER.text + ' => ' + toStringSafe(e)));
        throw e;
    }
    finally {
        await async(() => SPINNER.stop());
    }
}
exports.withSpinnerAsync = withSpinnerAsync;
/**
 * Writes an output message.
 *
 * @param {any} msg The message to write.
 */
function write(msg) {
    process.stdout.write(toStringSafe(msg));
}
exports.write = write;
/**
 * Writes an output message to error stream.
 *
 * @param {any} msg The message to write.
 */
function writeErr(msg) {
    process.stderr.write(toStringSafe(msg));
}
exports.writeErr = writeErr;
/**
 * Writes an (optional) output message to error stream and adds a new line.
 *
 * @param {any} [msg] The (optional) message to write.
 */
function writeErrLine(msg = '') {
    process.stderr.write(toStringSafe(msg) + os.EOL);
}
exports.writeErrLine = writeErrLine;
/**
 * Writes an (optional) output message and adds a new line.
 *
 * @param {any} [msg] The (optional) message to write.
 */
function writeLine(msg = '') {
    process.stdout.write(toStringSafe(msg) + os.EOL);
}
exports.writeLine = writeLine;
//# sourceMappingURL=util.js.map